/* $Id: glaceG.c,v 1.10 2000/09/14 06:07:15 acapnotic Exp $ */
/* :PREAMBLE: GlaceG.c
 *
 * Wrapper-specific Code:
 *           GIMP plug-in version
 *
 * Based on glaceT.c and glaceP.c by J.Alex Stark
 * g_ified by Kevin M. Turner
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.  See the file COPYING for details.
 *
 */

/* :HEADERS:
 */
#include <config.h>

#include <gtk/gtk.h> /* for gtk_preview manipulations */
#include <ctype.h>   /* for tolower in keymatch */
#include "glaceInt.h"
#include "preview.h"

extern GimpPDBStatusType* const gimp_ace_status;

static void repopulate_alpha(guchar* dst, 
			     const GlaceGimpDrawableInfo* d,
			     guint x, 
			     guint y, 
			     guint w, 
			     guint h);

/**************************************************************/

/* :*** Errors and Messages ***:

 * :FUNCTION: Glace_WError
 * Sets error status and bails. 
 */
int
Glace_WError (Glace_WData wData, char *argErrStr)
{
  if (wData == NULL)
    g_warning ("Error rountine called with invalid package data structure.");

  WRAPPER (errorValue) = GLACE_ERROR;

  *gimp_ace_status = GIMP_PDB_EXECUTION_ERROR;

  /* do both message and warning, as this should be seen by the user and
     be present in the error stream.  Do not use g_error, as that will
     abort rather than doing gimp_quit. */

  g_message("ACE: Fatal error, please notify <" "MAINTAINER_EMAIL" ">:\n"
	    "%s", argErrStr);
  g_warning("ACE: Fatal error, please notify <" "MAINTAINER_EMAIL" ">:\n"
	    "%s", argErrStr);
  /* gimp_quit(); */

  return GLACE_ERROR;
}

/*
 * :FUNCTION: Glace_WErrorValue
 */
int				/* Returns error flag */
Glace_WErrorValue ( Glace_WData wData )
{
  return WRAPPER (errorValue);
}



/*
 * :FUNCTION: Glace_WIsError
 */
gboolean			/* Returns boolean */
Glace_WIsError ( Glace_WData wData )
{
  return (WRAPPER (errorValue) == GLACE_ERROR);
}



/*
 * :FUNCTION: Glace_WMessage
 */
void
Glace_WMessage ( Glace_WData wData G_GNUC_UNUSED,
		 char *messageStr)
{
  g_print("%s", messageStr);
}


/**************************************************************/
/* :*** Wrapper-specific Setup ***:

 * :FUNCTION: Glace_WInit
 */
void
Glace_WInit (Glace_WData wData G_GNUC_UNUSED)
{
}



/**************************************************************/
/*
 * :FUNCTION: Glace_WWrapTell
 */
Glace_WrapTypes
Glace_WWrapTell()
{
  return GLACE_GIMP_VER;
}

/**************************************************************/
/*
 * :FUNCTION: Glace_WDataAlloc
 *
 */
Glace_WData
Glace_WDataAlloc ()
{
  return g_new0(GlaceGimpData,1);
}



/* :*** Argument Parsing and Warning ***:

 * :FUNCTION: Glace_WKeyMatch
 *
 * NB This is an independent rewrite of PBMPLUS routine,
 * just to be sure about copyright issues. [Alex]
 *
 * (Probably not necessary for GIMP version.  Feel free to substitute
 * pm_keymatch back in...) [kmt]
 */
int
Glace_WKeyMatch(
		char* str,
		char* keyword,
		int minchars
		)
{
  int i, slen;

  slen = strlen(str);

  if (slen < minchars)
    return 0;

  else if (slen > (int) strlen(keyword))
    return 0;

  else
    for (i=0; i<slen; i++)
      if (tolower(str[i]) != tolower(keyword[i]))
	return 0;
 
  return 1;
}

/**************************************************************/
/*
 * :FUNCTION: Glace_WUsage
 */
void
Glace_WUsage(
	     Glace_WData wData,
    char* usage
)
{
  Glace_WMessage(wData, usage);
  WRAPPER(errorValue) = GLACE_ERROR;
}

/* :*** Image Input and Output ***:

 * :FUNCTION: Glace_WOpenImage 
 */
void
Glace_WOpenImage(Glace_WData wData G_GNUC_UNUSED,
		 Glace_CfgInfo *cfgInfoPtr G_GNUC_UNUSED,
		 char *name G_GNUC_UNUSED,
		 Glace_ImageHandle *handlePtr G_GNUC_UNUSED )
{
  if(name==NULL) return;
  g_warning("Glace_WOpenImage(%s): function not implemented in glaceG.c",
	    name);
}

/**************************************************************/

/*
 * :FUNCTION: Glace_WPutImgStart
 */
void
Glace_WPutImgStart(Glace_WData wData, Glace_CfgInfo *cfgInfoPtr G_GNUC_UNUSED,
		   Glace_ImgArrays *imgArraysPtr G_GNUC_UNUSED)
{
  switch(WRAPPER(target)) 
    {
    case GLACE_GIMP_PixelRgn:
      {
	const int tile_height = gimp_tile_height();

	WRAPPER(output_buf)=g_new(GimpPixel, 
				  tile_height * WRAPPER(d)->gimpImgRowstride);

	WRAPPER(output_tile_row0)=0;
        GLACE_IMG(putImgRowPtr)=WRAPPER(output_buf);
      }
      break;
    case GLACE_GIMP_PREVIEW:
      break;
    default:
      g_warning("Lameness in WPutImgStart");
  } /* esac target */
}

/**************************************************************/
/*
 * :FUNCTION: Glace_WPutImgRowStart
 */
void
Glace_WPutImgRowStart(Glace_WData wData, 
		      Glace_CfgInfo *cfgInfoPtr G_GNUC_UNUSED,
		      Glace_ImgArrays *imgArraysPtr, 
		      int row G_GNUC_UNUSED)
{
  switch(WRAPPER(target)) 
    {
    case GLACE_GIMP_PixelRgn:
      break;
    case GLACE_GIMP_PREVIEW:
      GLACE_IMG(putImgRowPtr)=(Glace_Gray *)GLACE_IMG(tmpImgPtr);
      break;
    } /* esac target */
}

/**************************************************************/


 /*
  * :FUNCTION: Glace_WPutImgRowFinish
  *
  * Assumption: The first row is row = 0, not 1.
  * 
  * FIXME: This code needs the GTK lock.
  * 
  */
void
Glace_WPutImgRowFinish ( Glace_WData wData,
	                 Glace_CfgInfo *cfgInfoPtr G_GNUC_UNUSED,
			 Glace_ImgArrays * imgArraysPtr,
			 int row )
{
  switch(WRAPPER(target))
    {
    case GLACE_GIMP_PixelRgn:
      {
	static int tile_height = -1;
	if(tile_height == -1) tile_height = gimp_tile_height();

	/* This means to write the output one row of tiles at a time. */
	if(((row+1 + WRAPPER(d)->y0) % tile_height == 0) || 
	   (row+1 == WRAPPER(d)->h) /* or if this is the last row */)
	  {
	    GimpPixelRgn dest_rgn;
	    const int h = row+1 - WRAPPER(output_tile_row0);
	    const int y0 = WRAPPER(d)->y0 + WRAPPER(output_tile_row0);

	    if(WRAPPER(d)->has_alpha)
		repopulate_alpha(WRAPPER(output_buf),WRAPPER(d),
				 WRAPPER(d)->x0,y0,WRAPPER(d)->w,h);

	    gimp_pixel_rgn_init(&dest_rgn,
				WRAPPER(d)->drawable,
				WRAPPER(d)->x0, y0,
				WRAPPER(d)->w, h,
				TRUE, TRUE);
	    
	    gimp_pixel_rgn_set_rect(&dest_rgn, WRAPPER(output_buf),
				    WRAPPER(d)->x0, y0,
				    WRAPPER(d)->w, h);

	    GLACE_IMG(putImgRowPtr) = WRAPPER(output_buf);
	    WRAPPER(output_tile_row0) = row+1;
	  }
	else
	  {
	    /* Advance row pointer by rowstride */
	    GLACE_IMG(putImgRowPtr) += WRAPPER(d)->gimpImgRowstride;
	  }
      }
      break;
    case GLACE_GIMP_PREVIEW:
      gtk_preview_draw_row(wData->preview->preview,
			   GLACE_IMG(putImgRowPtr),
			   0, row, GLACE_IMG(viewCols));
      break;
    } /* esac target */
}


/**************************************************************/


/*
 * :FUNCTION: Glace_WPutImgFinish
 *
 * TODO: 
 *   Set UNDOABLE to FALSE on merge when Applying multiple times.
 * 
 * FIXME: This code needs the GTK lock.
 * 
 *
 */
void
Glace_WPutImgFinish ( Glace_WData wData, 
		      Glace_CfgInfo *cfgInfoPtr G_GNUC_UNUSED,
		      Glace_ImgArrays * imgArraysPtr G_GNUC_UNUSED)
{
  switch(WRAPPER(target)) 
    {
    case GLACE_GIMP_PixelRgn:
      {
	gimp_drawable_flush (WRAPPER(d)->drawable);
	gimp_drawable_merge_shadow (WRAPPER(d)->drawable->id, 
				    /*UNDOABLE*/TRUE);
	gimp_drawable_update (WRAPPER(d)->drawable->id, 
			      WRAPPER(d->x0), WRAPPER(d->y0),
			      WRAPPER(d->w), WRAPPER(d->h));

	g_free(WRAPPER(output_buf));
      }
      break;
    case GLACE_GIMP_PREVIEW:
      gtk_widget_draw(GTK_WIDGET(WRAPPER(preview)->preview),NULL);
      break;
    default:
      g_warning("Lameness in WPutImgFinish");
    } /* esac target */
}

/**************************************************************/

static void
repopulate_alpha(guchar* dst, const GlaceGimpDrawableInfo* d,
		 guint x, guint y, guint w, guint h)
{
  GimpPixel *src=d->gimpImgPtr 
    + (y - d->y0) * d->gimpImgRowstride 
    + (x - d->x0) * d->drawable->bpp;
  GimpPixel *rowsrc;
  guchar *rowdst;
  gint xx;

  g_return_if_fail(d->has_alpha);

  /* Advance to the alpha byte. */
  src += d->drawable->bpp -1;
  dst += d->drawable->bpp -1;
  
  for(;h>0;h--)
    {
      rowsrc = src;
      rowdst = dst;

      for(xx=w;xx>0;xx--,
	    rowsrc += d->drawable->bpp,
	    rowdst += d->drawable->bpp)
	{
	  *rowdst = *rowsrc;
	} /* next x */

      src += d->gimpImgRowstride;
      dst += d->gimpImgRowstride;

    } /* next y */
} /* repopulate_alpha */
