/* $Id: output.c,v 1.12 2000/09/14 07:10:26 acapnotic Exp $ */
/* This file should probably be folded back in to glace instead of being
 * GIMP-specific. */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.  See the file COPYING for details.
 *
 */

#include <config.h>
#include "output.h"

#include "color.h"              /* Contains constants for color transforms */
#include "glaceInt.h"           /* GlaceGimp_Output uses ACC_BITS, OUT_BITS */

#define CUBE(a)((a)*(a)*(a))

inline static int Ymax(double* xp, 
		       double* yp, 
		       int*   Yp, 
		       const gboolean add_white);

/* If DIAGNOSTIC is not defined, anything wrapped in DIAG() is a no-op. */
#ifdef DIAGNOSTIC
# define DIAG(X) X
#else
# define DIAG(X)
#endif

int
GlaceGimp_Output (Glace_WData wData,
		  Glace_CfgInfo *cfgInfoPtr,
		  Glace_ImgArrays *imgArraysPtr)
{
  Glace_MidGray temp_gray;
  const Glace_MidGray out_offset= 255 << (ACC_BITS - OUT_BITS);
  register int c;
  register int r;
  const Glace_MidGray *agP;
  const Glace_Gray *inP;
  Glace_Gray *ogP, tmph, tmpl;

  long temp_long;

  register int gstep;

  const gboolean grayout=(GLACE_CFG(chrome) != GLACE_CC);
  const gboolean doubleout=GLACE_CFG(doubleout);
  const gboolean doClip=GLACE_CFG(doClip);
  const Glace_ColorMethods colorMethod=GLACE_CFG(colorMethod);

  /* Once I figure out which options are staying and which are getting
     tossed out, I'll find a sane way to do this.  But for now, it's
     going to be a little sloppy and slightly silly. */
  const gboolean correctDark=
    GLACE_CFG(colorMethod) == GLACE_COLOR_SAT_MARTIN_DARK ||
    GLACE_CFG(colorMethod) == GLACE_COLOR_DESATURATE_DARK ||
    GLACE_CFG(colorMethod) == GLACE_COLOR_LIMIT_DARK ||
    GLACE_CFG(colorMethod) == GLACE_COLOR_SAT_ALEX_DARK;

#ifdef TROUBLEMASK
  const float MAX_LOG2 = logb(~((GimpPixel)0));
  const gboolean troublemasks = 
    GLACE_CFG(colorMethod) == GLACE_COLOR_DESATURATE_MASK;
#endif

#ifdef DIAGNOSTIC
  register guint count=0,count_gt1=0,count_gtmax=0,count_up=0,count_down=0;
  register guint count_gray=0, count_dark=0;
  register gulong total_plus_Y=0, total_minus_Y=0;
  static GTimer* timer=NULL;
  double elapsed=0.0;

  if(timer==NULL) timer=g_timer_new();
#endif

#ifdef THIS_MAKES_SENSE_TO_WHO
  /* 255 because that brings zero to the right (?) place and
     1 because that becomes 0.5, that is rounding by truncation. */
  if ((GLACE_CFG(chrome)==GLACE_CC) || (GLACE_CFG(doubleout)==TRUE))
    round_off=0;
  else
    round_off=1;

  out_offset = (255+round_off) << (ACC_BITS - OUT_BITS);
#endif /* makes sense? */
  
  if(doClip) g_warning("UNTESTED: Clipping field (doClip) has not been "
		       "tested in GlaceGimp_Output\n");

  if(doubleout && !grayout) 
    { 
      g_warning("GlaceGimp_Output doesn't even pretend to handle "
		"16-bit colour output.  Returning -1.\n");
      return -1;
    }

  if(doubleout) g_warning("UNTESTED: double-precision output has not been "
		       "tested in GlaceGimp_Output\n");			  

  /* We output row-by-row */
  Glace_WPutImgStart(wData,cfgInfoPtr,imgArraysPtr);

  DIAG(g_timer_start(timer));

  agP = GLACE_IMG(outAccImgPtr);
  /* Skip the rows before the view starts. */
  agP += (GLACE_IMG(viewY0) - GLACE_IMG(windowY0)) * GLACE_IMG(cols);
  /* Skip the columns before the view starts. */
  agP += GLACE_IMG(viewX0) - GLACE_IMG(windowX0);
  for (r = 0; r < GLACE_IMG(viewRows); r++)
    {
      Glace_WPutImgRowStart(wData, cfgInfoPtr, imgArraysPtr, r);

      if (grayout)
	{
	  if(!doubleout)
	    {
	      gstep=GLACE_IMG(pixelBytePad);
	      if (!doClip)
		{
		  for (c=GLACE_IMG(viewCols), ogP = GLACE_IMG(putImgRowPtr);
		       c >0 ; c--, ogP++, ogP +=gstep, agP++)
		    {
		      temp_gray = (*agP + out_offset)
			>> (ACC_BITS - OUT_BITS + 1);
		      /*temp_gray = (Glace_MidGray) (((float) *agP)*tmpy +tmpz);*/
		      *ogP = temp_gray > 255 ? 255 : temp_gray < 0 ? 0 : temp_gray;
		      DIAG(count++;)
		    }
		}
#ifdef DIAGNOSTIC
	      else /* doClip - FIXME - untested */
		{
		  for (c = GLACE_IMG(cols), ogP = GLACE_IMG(putImgRowPtr);
		       c >0 ; c--, ogP++, ogP +=gstep, agP++)
		    {
		      temp_gray = (*agP + out_offset)
			>> (ACC_BITS - OUT_BITS + 1);
		      /*temp_gray = (Glace_MidGray) (((float) *agP)*tmpy +tmpz);*/
		      *ogP = temp_gray > 255 ? 255 : temp_gray < 0 ? 0 : 127;
		    } /* next pixel */
		} /* endif !doClip */
#endif
	    } 
	  else /* doubleout - FIXME - untested */
	    {
	      gstep = 3+GLACE_IMG(pixelBytePad);
	      for (c = GLACE_IMG(viewCols), ogP = GLACE_IMG(putImgRowPtr);
		     c >0; c--, ogP+= gstep,++agP )
		{
		  temp_long = (*agP + out_offset)
		    << (8-(ACC_BITS - OUT_BITS + 1));
		  temp_long = temp_long > 65535 ? 65535 : 
		    temp_long < 0 ? 0 : temp_long;
		  BYTE_SPLIT(temp_long, tmph, tmpl);
		  GLACE_ASSIGN(
			       *((Glace_Pixel *)ogP),
			       tmph,tmph,tmpl);
		} /* next pixel */
	    } /* endif doubleout */
	} /* endif grayout */
      else /* !grayout */
	{
	  /* FIXME: Nothing in here should depend on WRAPPER */
	  const int instep = WRAPPER(d)->drawable->bpp;
	  gstep = 3 + GLACE_IMG(pixelBytePad);
	  for ( c = GLACE_IMG(viewCols),
		  ogP = GLACE_IMG(putImgRowPtr),
		  inP = (WRAPPER(d)->gimpImgPtr + 
			 (r+GLACE_IMG(viewY0)) * WRAPPER(d)->gimpImgRowstride +
			 GLACE_IMG(viewX0) * WRAPPER(d)->drawable->bpp);
		c != 0;
		c--, ogP+=gstep, inP+=instep, ++agP )
	    {
	      register int vR=inP[0];
	      register int vG=inP[1];
	      register int vB=inP[2];
	      const int beta0 = MAX(MAX(vR, vG), vB);
	      int vY2 =  (*agP + out_offset) / (1<<(ACC_BITS - OUT_BITS + 1));
	      /* We do "a / (1 << b)" instead of "a >> b" because the behaviour
		 of "a >> b" is implementation dependant when a is negative.
		 We could cook up an autoconf test for this if we were really
		 worried about that extra division... */

	      if((colorMethod == GLACE_COLOR_GRAY) || (vR==vG && vR==vB) || 
		 (vR<(1/Y_r + 0.5) && vG < (1/Y_g + 0.5) && vB < (1/Y_b +0.5)))
		{
		  vR=vG=vB=CLAMP0255(vY2);
		  DIAG(count_gray++);
		}
	      else
		switch(colorMethod)
		  {
		  case GLACE_COLOR_LIMIT:
		  case GLACE_COLOR_LIMIT_DARK:
		  case GLACE_COLOR_DESATURATE:
		  case GLACE_COLOR_DESATURATE_DARK:
		    {
		      int c_changed = 0;
		      int       vX = X_r * vR + X_g * vG + X_b * vB + 0.5;
		      const int vY1= Y_r * vR + Y_g * vG + Y_b * vB + 0.5;
		      int       vZ = Z_r * vR + Z_g * vG + Z_b * vB + 0.5;
		      double vx,vy;

		      if(vY2 > vY1)
			{
			  vx = (double)vX /(vX+vY1+vZ);
			  vy = (double)vY1/(vX+vY1+vZ);
			  c_changed = Ymax(&vx,&vy,&vY2,
					   (colorMethod != 
					   GLACE_COLOR_LIMIT) && 
					   (colorMethod != GLACE_COLOR_LIMIT_DARK));

			  DIAG(if (c_changed) count_gtmax++);
			  DIAG(count_up++; total_plus_Y += vY2 - vY1);

#ifdef TROUBLEMASK			  
			  if(troublemask)
			    {
			      /* Dark colors being lightened: */
			      /* The problem with dark colors is that
				 there aren't enough bits to
				 accurately determine chromacity.
				 Take the highest bit set as a measure
				 of this. */
			      /* note: you could use ffs() instead of log2 here
				 if it becomes a speed concern. */
			      float mask_exp = MAX_LOG2 - 
				log2(MAX(MAX(vR,vG),vB));
			      *dark_mask = (*mask_exp < MAX_LOG2 / 2) ? 0 : 
				(*dark_exp/(MAX_LOG/2) - 1) * 255;
			    } /* endif troublemask */
#endif /* TROUBLEMASK */
			}
		      else
			{
			  if(vY2 < 0) vY2 = 0;
			  DIAG(if(vY2<vY1){count_down++;total_minus_Y+=vY1-vY2;});

#ifdef TROUBLEMASK
			  if(troublemask)
			    {
			      /* Light colors: */
			      /* The problem with light colors isn't quite the
				 same, but we'll mask it similarly until
				 something better comes along. */
			      float mask_exp = MAX_LOG2 -
				log2(MAX(MAX(~(guchar)vR,~(guchar)vG),
					 ~(guchar)vB));
			      *light_mask = (*mask_exp < MAX_LOG2 / 2) ? 0 : 
				(*dark_exp/(MAX_LOG/2) - 1) * 255;
			    } /* endif troublemask */
#endif /* TROUBLEMASK */
			}
		      
		      if(c_changed)
			{
			  vX = vY2 * vx/vy;
			  vZ = vY2 * (1.0 - vx - vy) / vy;

#ifdef TROUBLEMASK
			  if(troublemask)
			    {
			      /* This mask is directly related to the number
				 of luminosity-units by which Y2 exceeded
				 the gamut. */
			      *gamut_mask = MIN(4 * c_changed, 255);
			    }
#endif /* TROUBLEMASK */
			} 
		      else 
			{
			  /* simplifies to: */
			  vX = vX * vY2 / vY1;
			  vZ = vZ * vY2 / vY1;
			}
		      
		      vR = CLAMP0255(R_x*vX + R_y*vY2 + R_z*vZ + 0.5);
		      vG = CLAMP0255(G_x*vX + G_y*vY2 + G_z*vZ + 0.5);
		      vB = CLAMP0255(B_x*vX + B_y*vY2 + B_z*vZ + 0.5);

		      if(correctDark && beta0 < 127)
			{
			  float correctdark = CUBE ((255 - beta0) / 255.0);
			  vR = vR * (1.0 - correctdark) + correctdark * vY2;
			  vG = vG * (1.0 - correctdark) + correctdark * vY2;
			  vB = vB * (1.0 - correctdark) + correctdark * vY2;

			  DIAG(count_dark++);
			}
		    }
		    break;
		  case GLACE_COLOR_SAT_ALEX:
		  case GLACE_COLOR_SAT_ALEX_DARK:
		  case GLACE_COLOR_SAT_MARTIN:
		  case GLACE_COLOR_SAT_MARTIN_DARK:
		    {
		      const double YMAP_ADD=1.0;
		      const Glace_Gray mmaxval = GLACE_MAXMAXGRAY;
		      const gint32 vY1=Y_r * vR + Y_g * vG + Y_b * vB;
		      const int vR0=vR;
		      const int vG0=vG;
		      const int vB0=vB;		      
		      double lambda;
		      int alpha, beta;
		      
		      vY2 = CLAMP0255(vY2);
		      
#ifdef DIAGNOSTIC
		      if(vY2 > vY1) 
			{
			  if(vY2>255) count_gt1++;
			  count_up++;
			  total_plus_Y += vY2 - vY1;
			} 
		      else if (vY2 < vY1) 
			{
			  count_down++;
			  total_minus_Y += vY1 - vY2;
			}			
#endif /* DIAGNOSTIC */
		      
		      vR -= vY1;
		      vG -= vY1;
		      vB -= vY1;

		      beta = MAX(MAX(vR, vG), vB);
		      alpha = abs(MIN(MIN(vR, vG), vB));
		      
		      lambda = MIN( beta*vY2, alpha*(mmaxval-vY2) ) /
			(YMAP_ADD+MIN( beta*vY1, alpha*(mmaxval-vY1) ) );
		      
		      vR = vY2 + lambda* vR;
		      vG = vY2 + lambda* vG;
		      vB = vY2 + lambda* vB;
		      
		      if(colorMethod==GLACE_COLOR_SAT_MARTIN || 
			 colorMethod==GLACE_COLOR_SAT_MARTIN_DARK)
			{
			  float correctlight;
			  double vSum = (vR0 + vG0 + vB0)/(255.0*3);

                          correctlight = SQR(SQR(SQR (vSum)));

                          vR = vR * (1.0 - correctlight) + correctlight * vY2;
			  vG = vG * (1.0 - correctlight) + correctlight * vY2;
			  vB = vB * (1.0 - correctlight) + correctlight * vY2;
			}

		      if(correctDark && beta0 < 127)
			{
			  float correctdark = CUBE ((255 - beta0) / 255.0);
			  vR = vR * (1.0 - correctdark) + correctdark * vY2; 
			  vG = vG * (1.0 - correctdark) + correctdark * vY2;
			  vB = vB * (1.0 - correctdark) + correctdark * vY2;
			  DIAG(count_dark++);
			}
		    }
		    break;
		  case GLACE_COLOR_GRAY:
		    /* is handled before this switch */
		  case GLACE_COLOR_LAST:
		    /* just a placeholder */
		    break;
		  } /* esac colorMethod */

	      GLACE_ASSIGN(*((Glace_Pixel *)ogP),
			   vR, vG, vB);
	      DIAG(count++);
	    } /* next pixel */
	} /* endif !grayout */      
      Glace_WPutImgRowFinish(wData, cfgInfoPtr, imgArraysPtr, r);

      /* complete rowstride */
      agP += GLACE_IMG(cols) - GLACE_IMG(viewCols);

    } /* next row */

#ifdef DIAGNOSTIC
  elapsed=g_timer_elapsed(timer,NULL);
  g_timer_stop(timer);

  if(GLACE_CFG(verbose))
    {
      g_print("Pixels :\t%d\n" 
	      "Y2 > Y1:\t%d\t(%4.1f%% of total)\n"
	      "Y2 > YMax:\t%d\t(%4.1f%% of Y2 > Y1)\n"
	      "Y2 > 1 :\t%d\t(%4.1f%% of Y2 > Y1)\n",
	      count,
	      count_up, 100 * (double)count_up / count,
	      count_gtmax, 100 * (double)count_gtmax / count_up,
	      count_gt1, 100 *(double)count_gt1 / count_up);
      g_print("avg rise:\t% 5f\n" 
	      "avg drop:\t% 5f\n"
	      "overall :\t%+5f\n",
	      (double)total_plus_Y/(double)count_up/255.0,
	      (double)total_minus_Y/(double)count_down/255.0,
	      (double)((signed)total_plus_Y - (signed)total_minus_Y)/
	      (double)count/255.0);
      g_print("fast grays:\t%d\t(%4.1f%% of total)\n"
	      "correctdark:\t%d\t(%4.1f%% of total)\n",
	      count_gray,100*count_gray/(double)count,
	      count_dark,100*count_dark/(double)count
	      );
      g_print("Output time: %f seconds\n"
	      "Milliseconds per pixel: %f\n",
	      elapsed, 1000.0 * 1000.0 * elapsed/count);
    } /* endif GLACE_CFG(verbose) */
#endif /* DIAGNOSTIC */
  
  Glace_WPutImgFinish(wData, cfgInfoPtr, imgArraysPtr);
  
  return Glace_WErrorValue(wData);
}

/* This makes more sense if you look at the graph.
   run 'gnuplot < doc/max-luminosity.gnuplot' for a visual. */
/* x and y are chromacity coordinates. 
   Y is luminosity scaled 0-255 */
/* and no, this hasn't yet been tuned for speed. */
inline static int
Ymax(double* const xp, double* const yp, int* const Yp, 
     const gboolean add_white)
{
  const int Y = *Yp;
  const double y = *yp;
  const double x = *xp;
  gint Ymax=255;
  typedef enum { RED_ROOF, GREEN_ROOF, BLUE_ROOF } RoofColor;
  RoofColor shingle;

  if(y > y_White) /* above Cyan-White line */
    {
      if(x > x_White) /* right of White */
	{
	  /* below white-yellow line? */
	  if((y - y_White) < 
	     ((y_Yellow - y_White) * (x - x_White) / (x_Yellow - x_White)))
	    {
	      shingle = RED_ROOF;
	    } else {
	      shingle = GREEN_ROOF;
	    } /* end white-yellow-line */
	} else {
	  shingle = GREEN_ROOF;
	} /* endif left-of-white */
    }
  else  /* Below Cyan-White line */
    {
      if ((x - x_White) < /* left of White-Purple line */
	  ((y - y_White) * (x_Purple-x_White)/(y_Purple-y_White)))
	{
	  shingle = BLUE_ROOF;
	} else {
	  shingle = RED_ROOF;
	}
    } /* endif below cyan-white */

  switch (shingle)
    {
    case RED_ROOF:
      Ymax= 255.0 /(R_y + R_x*(x)/(y) + R_z * (1.0-x-y) / y);
      break;
    case GREEN_ROOF:
      Ymax= 255.0 /(G_y + G_x*(x)/(y) + G_z * (1.0-x-y) / y);
      break;
    case BLUE_ROOF:
      Ymax= 255.0 /(B_y + B_x*(x)/(y) + B_z * (1.0-x-y) / y);
      break;
    }

  if(Y <= Ymax)
    {
      /* nop, everything remains unchanged. */
      return FALSE;
    } 
  else if(!add_white) /* Y > Ymax */
    {
      /* x and y unchanged */
      *Yp=Ymax;
      return FALSE;
    }
  else 
    if (Y >= 255) /* add_white && Y > 100% */
    {
      *xp=x_White;
      *yp=y_White;
      *Yp=255;
      return (Y - Ymax);
    }
  else /* add_white && 255 > Y > Ymax */
    {
      double x2, y2=0;
      const double dxdy = (x_White - x) / (y_White - y);
      switch (shingle)
	{
	case RED_ROOF:
	  y2=(x * (R_x - R_z) + y * dxdy * (R_z-R_x) + R_z) /
	     (255.0/Y + dxdy * (R_z - R_x) - R_y + R_z);
	  break;
	case GREEN_ROOF:
	  y2=(x * (G_x - G_z) + y * dxdy * (G_z-G_x) + G_z) /
	     (255.0/Y + dxdy * (G_z - G_x) - G_y + G_z);
	  break;
	case BLUE_ROOF:
	  y2=(x * (B_x - B_z) + y * dxdy * (B_z-B_x) + B_z) /
	     (255.0/Y + dxdy * (B_z - B_x) - B_y + B_z);
	  break;
	}
      x2 = x + dxdy * (y2 - y);
  
      *xp = x2;
      *yp = y2;
      
      /* Y is unchanged. */
      return (Y - Ymax); /* chromacity did change */
    } /* endif addwhite */
}

/* For emacs:
Local Variables:
mode:c
c-file-style:"gnu"
End:
*/
