/*********************************************************************************/
/* LIC 0.50 -- image filter plug-in for The Gimp program                         */
/* Copyright (C) 2001/02 Frank Lauterwald                                        */
/*===============================================================================*/
/* E-mail: frankl@gmx.net                                                        */
/* You can contact the original The Gimp authors at gimp@xcf.berkeley.edu        */
/*===============================================================================*/
/* This program is free software; you can redistribute it and/or modify it under */
/* the terms of the GNU General Public License as published by the Free Software */
/* Foundation; either version 2 of the License, or (at your option) any later    */
/* version.                                                                      */
/*===============================================================================*/
/* This program is distributed in the hope that it will be useful, but WITHOUT   */
/* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS */
/* FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.*/
/*===============================================================================*/
/* You should have received a copy of the GNU General Public License along with  */
/* this program; if not, write to the Free Software Foundation, Inc., 675 Mass   */
/* Ave, Cambridge, MA 02139, USA.                                                */
/*===============================================================================*/
/* In other words, you can't sue me for whatever happens while using this ;)     */
/*********************************************************************************/
/* CHANGELOG									 */
/* Date		Version		Comment						 */
/* 2001 Oct. 1,    0.30		first "release"					 */
/* 2002 Jan 17	   0.35		started adding the noise stuff from Toms filter  */
/* 2002 Jan. 26,   0.50		release of first beta version			 */
/*********************************************************************************/

/* This file is able to calculate a result image provided all options are set correctly before */
/* in case the texture is to be taken from white noise an extra image with white noise */
/* has to be prepared before. This image has to have the correct size (i.e. >= effectimage) */

/* I think a few remarks are needed as of how we deal with different bit depths:       */
/* 1. We use a custom block for buffering data. variables.pixelregiondata is a pointer */
/* to this block. FIXME: describe */

#include "lic_algorithm.h"
#include <libgimp/gimp.h>
#include "../lic_globals.h"
#include "../lic_color.h"
#include "../lic_noise.h"
#include "../filterfunction/lic_filter.h"
#include "lic_algorithm_include.h"


/* just a small macro */
#define min(a,b) (a>b?b:a)
#define max(a,b) (a<b?b:a)


void swapin_input_block(gint x1, gint y1, gint x2, gint y2)
{		//fprintf(stderr, "swapping in input %i %i - %i %i \n", x1,y1,x2,y2);
		gimp_pixel_rgn_get_rect(&variables.input_region, (guchar*)variables.inputpixelregiondata, x1, y1, x2-x1+1, y2-y1+1);
		variables.input_swappedin_x1=x1;	variables.input_swappedin_y1=y1;	variables.input_swappedin_x2=x2;	variables.input_swappedin_y2=y2;
}

void swapin_output_block(gint x1, gint y1, gint x2, gint y2)
{		//fprintf(stderr, "swapping in output %i %i - %i %i \n", x1,y1,x2,y2);
		gimp_pixel_rgn_get_rect(&variables.output_region, (guchar*)variables.outputpixelregiondata, x1, y1, x2-x1+1, y2-y1+1);

		/* make the new coordinates known as swapped-in */
		variables.output_swappedin_x1=x1;	variables.output_swappedin_y1=y1;	variables.output_swappedin_x2=x2;	variables.output_swappedin_y2=y2;
}

void swapout_output_block()
{		//fprintf(stderr, "swapping out output \n");
		gimp_pixel_rgn_set_rect(&variables.output_region, (guchar*)variables.outputpixelregiondata, variables.output_swappedin_x1, variables.output_swappedin_y1, variables.output_swappedin_x2-variables.output_swappedin_x1+1, variables.output_swappedin_y2-variables.output_swappedin_y1+1);
}


void lic_compute_image()
{ /* loops for x and y: swap in a block and do calculations for it */

	int leftedge, topedge;
	int swapin_x1, swapin_x2, swapin_y1, swapin_y2;
	int withinblock_x, withinblock_y;
	int numberofcalculatedpixels, numberofpixelstocalculate;
	lic_calculatepixelproc calculatepixel;
	int maxcounterx, maxcountery; /* the values to which we will count up in the main loop */

	/* set the correct procedure to compute a pixel depending on chosen algorithm and color model	*/
	switch (gimp_drawable_get(parameters.options.textureImageDrawable)->bpp)
	{	case 1:	{
				if (parameters.options.getTextureFrom == GETTEXTUREFROM_IMAGE) {
					if (parameters.algorithm.algorithm == ALGORITHM_LICEULER)
					{	if (parameters.filterfunction.filterfunction == FILTERFUNCTION_BOX)
							{   calculatepixel=algorithm_lic_euler_compute_pixel_gray_blockfilter;
							}
						else
							{    calculatepixel=algorithm_lic_euler_compute_pixel_gray_anyfilter;   }
					}
					if (parameters.algorithm.algorithm == ALGORITHM_DDAEULER)
					{	if (parameters.filterfunction.filterfunction == FILTERFUNCTION_BOX)
							{   calculatepixel=algorithm_dda_euler_compute_pixel_gray_blockfilter;  }
						else
							{    calculatepixel=algorithm_dda_euler_compute_pixel_gray_anyfilter; }
					}
					if (parameters.algorithm.algorithm == ALGORITHM_LICCABRAL)
					{	if (parameters.filterfunction.filterfunction == FILTERFUNCTION_BOX)
							{    calculatepixel=algorithm_lic_cabral_compute_pixel_gray_blockfilter;  }
						else
							{    calculatepixel=algorithm_lic_cabral_compute_pixel_gray_anyfilter;  }
					}
					if (parameters.algorithm.algorithm == ALGORITHM_DDACABRAL)
					{	if (parameters.filterfunction.filterfunction == FILTERFUNCTION_BOX)
							{   calculatepixel=algorithm_dda_cabral_compute_pixel_gray_blockfilter;  }
						else
							{    calculatepixel=algorithm_dda_cabral_compute_pixel_gray_anyfilter;  }
					}
				}
				if (parameters.options.getTextureFrom == GETTEXTUREFROM_NOISE) {
					if (parameters.algorithm.algorithm == ALGORITHM_LICEULER) {
						calculatepixel=algorithm_lic_euler_compute_pixel_gray_anyfilter_noise;
					}
					if (parameters.algorithm.algorithm == ALGORITHM_DDAEULER) {
						calculatepixel=algorithm_dda_euler_compute_pixel_gray_anyfilter_noise;
					}
				}
				break;
			}
		case 3:	{	if (parameters.options.getTextureFrom == GETTEXTUREFROM_IMAGE) {
					if (parameters.algorithm.algorithm == ALGORITHM_LICEULER)
					{	if (parameters.filterfunction.filterfunction == FILTERFUNCTION_BOX)
							{   calculatepixel=algorithm_lic_euler_compute_pixel_rgb_blockfilter;  }
						else
							{    calculatepixel=algorithm_lic_euler_compute_pixel_rgb_anyfilter;   }
					}
					if (parameters.algorithm.algorithm == ALGORITHM_DDAEULER)
					{	if (parameters.filterfunction.filterfunction == FILTERFUNCTION_BOX)
							{   calculatepixel=algorithm_dda_euler_compute_pixel_rgb_blockfilter;  }
						else
							{    calculatepixel=algorithm_dda_euler_compute_pixel_rgb_anyfilter; }
					}
					if (parameters.algorithm.algorithm == ALGORITHM_LICCABRAL)
					{	if (parameters.filterfunction.filterfunction == FILTERFUNCTION_BOX)
							{    calculatepixel=algorithm_lic_cabral_compute_pixel_rgb_blockfilter;  }
						else
							{    calculatepixel=algorithm_lic_cabral_compute_pixel_rgb_anyfilter;  }
					}
					if (parameters.algorithm.algorithm == ALGORITHM_DDACABRAL)
					{	if (parameters.filterfunction.filterfunction == FILTERFUNCTION_BOX)
							{   calculatepixel=algorithm_dda_cabral_compute_pixel_rgb_blockfilter;  }
						else
							{    calculatepixel=algorithm_dda_cabral_compute_pixel_rgb_anyfilter;  }
					}
				}
				if (parameters.options.getTextureFrom == GETTEXTUREFROM_NOISE) {
						if (parameters.algorithm.algorithm == ALGORITHM_LICEULER) {
							calculatepixel=algorithm_lic_euler_compute_pixel_rgb_anyfilter_noise;
						}
						if (parameters.algorithm.algorithm == ALGORITHM_DDAEULER) {
							calculatepixel=algorithm_dda_euler_compute_pixel_rgb_anyfilter_noise;
						}
				}
				break;
			}
	}
	/* set the correct filter function	 */
	switch (parameters.filterfunction.filterfunction)
	{	case FILTERFUNCTION_BOX: 	variables.filter = filter_blockfilter;
						variables.filter_integrate = filter_blockfilter_integral;
						break;
 		case FILTERFUNCTION_SMOOTHBOX: 	variables.filter = filter_smoothblockfilter;
						variables.filter_integrate = filter_smoothblockfilter_integral;
						break;
		case FILTERFUNCTION_ASTROID: 	variables.filter = filter_astroidfilter;
						variables.filter_integrate = filter_astroidfilter_integral;
						break;
 		case FILTERFUNCTION_TRIANGLE: 	variables.filter = filter_trianglefilter;
						variables.filter_integrate = filter_trianglefilter_integral;
						break;
		case FILTERFUNCTION_OTHER:	variables.filter = filter_otherfilter;
						break;
	}


	/* get regions for input and output	*/
	if (!(variables.inputpixelregiondata=malloc(BLOCKSIZE*BLOCKSIZE*4)))
	{	fprintf(stderr,"lic_compute_image : cannot allocate memory for inputpixelregiondata \n"); exit(EXIT_FAILURE);
	}
	if (!(variables.outputpixelregiondata=malloc(BLOCKSIZE*BLOCKSIZE*4)))
	{	fprintf(stderr,"lic_compute_image : cannot allocate memory for outputpixelregiondata \n"); exit(EXIT_FAILURE);
	}

	gimp_pixel_rgn_init(&variables.output_region, gimp_drawable_get(parameters.options.resultImageDrawable), 0,0,variables.width, variables.height, TRUE, TRUE);
	gimp_pixel_rgn_init(&variables.input_region, gimp_drawable_get(parameters.options.textureImageDrawable), 0,0,variables.width, variables.height, FALSE, FALSE);

	/* initialize some stuff */
	if (parameters.options.getTextureFrom == GETTEXTUREFROM_NOISE) {
		oldlic_noise_init();
	}
	numberofpixelstocalculate = variables.height*variables.width;
	numberofcalculatedpixels  = 0;
	gimp_progress_init("Van Gogh (Lic2)");
	VOUT("computing image");
	for (topedge=0; (topedge) < variables.height; topedge+=BLOCKSIZE-MAXFILTERLEN-MAXFILTERLEN)
	{
	        for (leftedge=0; (leftedge) < variables.width; leftedge+=BLOCKSIZE-MAXFILTERLEN-MAXFILTERLEN )
		{
                	/* calculate how large the swapped-in block will actually be */
                	swapin_x1 = leftedge-MAXFILTERLEN;
                	swapin_x2 = min(swapin_x1+BLOCKSIZE-1, variables.width-1);
			if (swapin_x1<0) { swapin_x1=0; }
                	swapin_y1 = topedge-MAXFILTERLEN;
                	swapin_y2 = min(swapin_y1+BLOCKSIZE-1, variables.height-1);
			if (swapin_y1<0) { swapin_y1=0; }


			/* -- calculate how large the swapped-in block will actually be*/
			variables.inputblockwidth   = swapin_x2-swapin_x1+1;
			variables.inputblockheight  = swapin_y2-swapin_y1+1;
			variables.outputblockwidth  = min(BLOCKSIZE-MAXFILTERLEN-MAXFILTERLEN, variables.width-leftedge);
			variables.outputblockheight = min(BLOCKSIZE-MAXFILTERLEN-MAXFILTERLEN, variables.height-topedge);
			swapin_input_block(swapin_x1, swapin_y1, swapin_x2, swapin_y2);
			swapin_output_block(leftedge, topedge, leftedge+variables.outputblockwidth-1, topedge+variables.outputblockheight-1);

			//fprintf(stderr, "blockwidth/height: %i %i \n",variables.inputblockwidth, variables.inputblockheight);
			//fprintf(stderr, "calculating: %i %i bis %i %i \n\n",leftedge, topedge, leftedge+variables.outputblockwidth-1, topedge+variables.outputblockheight-1);
			for (withinblock_y=0; withinblock_y < variables.outputblockheight; withinblock_y++)
                	{	for (withinblock_x=0; withinblock_x < variables.outputblockwidth; withinblock_x++)
	                	{	calculatepixel(withinblock_x, withinblock_y, withinblock_x+leftedge, withinblock_y+topedge);
                        	}
                        	numberofcalculatedpixels+=variables.outputblockwidth;
                        	/* progress_updaten */
				if (!(withinblock_y%10)) { gimp_progress_update((gfloat)numberofcalculatedpixels/(gfloat)numberofpixelstocalculate); }
			}
			swapout_output_block();
		}
	}
	/* swap out the last block then free temporary memory */
	free(variables.outputpixelregiondata);
	free(variables.inputpixelregiondata);
	if (parameters.options.getTextureFrom == GETTEXTUREFROM_NOISE) {
		oldlic_noise_done();
	}
}

/* now come the millions of functions for computing output pixels.
// we need to define the following:
// _filter_blockfilter_ or _filter_anyfilter_
// _colormodel_rgb_ or _colormodel_rgba_ or _colormodel_gray_ or _colormodel_graya_
// _algorithm_lic_ or _algorithm_dda_
// _noise_ or not
// (most combinations of the above) (all combinations except that _noise_ and _filter_blockfilter_ never go together */

#define _filter_blockfilter_
#define _algorithm_lic_
#define _colormodel_rgb_
#include "lic_algorithm_include.c"
#undef  _filter_blockfilter_
#undef  _algorithm_lic_
#undef  _colormodel_rgb_

#define _filter_blockfilter_
#define _algorithm_lic_
#define _colormodel_rgba_
#include "lic_algorithm_include.c"
#undef  _filter_blockfilter_
#undef  _algorithm_lic_
#undef  _colormodel_rgba_

#define _filter_blockfilter_
#define _algorithm_lic_
#define _colormodel_gray_
#include "lic_algorithm_include.c"
#undef  _filter_blockfilter_
#undef  _algorithm_lic_
#undef  _colormodel_gray_

#define _filter_blockfilter_
#define _algorithm_lic_
#define _colormodel_graya_
#include "lic_algorithm_include.c"
#undef  _filter_blockfilter_
#undef  _algorithm_lic_
#undef  _colormodel_graya_

#define _filter_blockfilter_
#define _algorithm_dda_
#define _colormodel_rgb_
#include "lic_algorithm_include.c"
#undef  _filter_blockfilter_
#undef  _algorithm_dda_
#undef  _colormodel_rgb_

#define _filter_blockfilter_
#define _algorithm_dda_
#define _colormodel_rgba_
#include "lic_algorithm_include.c"
#undef  _filter_blockfilter_
#undef  _algorithm_dda_
#undef  _colormodel_rgba_

#define _filter_blockfilter_
#define _algorithm_dda_
#define _colormodel_gray_
#include "lic_algorithm_include.c"
#undef  _filter_blockfilter_
#undef  _algorithm_dda_
#undef  _colormodel_gray_

#define _filter_blockfilter_
#define _algorithm_dda_
#define _colormodel_graya_
#include "lic_algorithm_include.c"
#undef  _filter_blockfilter_
#undef  _algorithm_dda_
#undef  _colormodel_graya_




#define _filter_anyfilter_
#define _algorithm_lic_
#define _colormodel_rgb_
#include "lic_algorithm_include.c"
#undef  _filter_anyfilter_
#undef  _algorithm_lic_
#undef  _colormodel_rgb_

#define _filter_anyfilter_
#define _algorithm_lic_
#define _colormodel_rgba_
#include "lic_algorithm_include.c"
#undef  _filter_anyfilter_
#undef  _algorithm_lic_
#undef  _colormodel_rgba_

#define _filter_anyfilter_
#define _algorithm_lic_
#define _colormodel_gray_
#include "lic_algorithm_include.c"
#undef  _filter_anyfilter_
#undef  _algorithm_lic_
#undef  _colormodel_gray_

#define _filter_anyfilter_
#define _algorithm_lic_
#define _colormodel_graya_
#include "lic_algorithm_include.c"
#undef  _filter_anyfilter_
#undef  _algorithm_lic_
#undef  _colormodel_graya_

#define _filter_anyfilter_
#define _algorithm_dda_
#define _colormodel_rgb_
#include "lic_algorithm_include.c"
#undef  _filter_anyfilter_
#undef  _algorithm_dda_
#undef  _colormodel_rgb_

#define _filter_anyfilter_
#define _algorithm_dda_
#define _colormodel_rgba_
#include "lic_algorithm_include.c"
#undef  _filter_anyfilter_
#undef  _algorithm_dda_
#undef  _colormodel_rgba_

#define _filter_anyfilter_
#define _algorithm_dda_
#define _colormodel_gray_
#include "lic_algorithm_include.c"
#undef  _filter_anyfilter_
#undef  _algorithm_dda_
#undef  _colormodel_gray_

#define _filter_anyfilter_
#define _algorithm_dda_
#define _colormodel_graya_
#include "lic_algorithm_include.c"
#undef  _filter_anyfilter_
#undef  _algorithm_dda_
#undef  _colormodel_graya_



#define _noise_

#define _filter_anyfilter_
#define _algorithm_lic_
#define _colormodel_rgb_
#include "lic_algorithm_include.c"
#undef  _filter_anyfilter_
#undef  _algorithm_lic_
#undef  _colormodel_rgb_

#define _filter_anyfilter_
#define _algorithm_lic_
#define _colormodel_rgba_
#include "lic_algorithm_include.c"
#undef  _filter_anyfilter_
#undef  _algorithm_lic_
#undef  _colormodel_rgba_

#define _filter_anyfilter_
#define _algorithm_lic_
#define _colormodel_gray_
#include "lic_algorithm_include.c"
#undef  _filter_anyfilter_
#undef  _algorithm_lic_
#undef  _colormodel_gray_

#define _filter_anyfilter_
#define _algorithm_lic_
#define _colormodel_graya_
#include "lic_algorithm_include.c"
#undef  _filter_anyfilter_
#undef  _algorithm_lic_
#undef  _colormodel_graya_

#define _filter_anyfilter_
#define _algorithm_dda_
#define _colormodel_rgb_
#include "lic_algorithm_include.c"
#undef  _filter_anyfilter_
#undef  _algorithm_dda_
#undef  _colormodel_rgb_

#define _filter_anyfilter_
#define _algorithm_dda_
#define _colormodel_rgba_
#include "lic_algorithm_include.c"
#undef  _filter_anyfilter_
#undef  _algorithm_dda_
#undef  _colormodel_rgba_

#define _filter_anyfilter_
#define _algorithm_dda_
#define _colormodel_gray_
#include "lic_algorithm_include.c"
#undef  _filter_anyfilter_
#undef  _algorithm_dda_
#undef  _colormodel_gray_

#define _filter_anyfilter_
#define _algorithm_dda_
#define _colormodel_graya_
#include "lic_algorithm_include.c"
#undef  _filter_anyfilter_
#undef  _algorithm_dda_
#undef  _colormodel_graya_

#undef _noise_
