/*********************************************************************************/
/* LIC 0.50 -- image filter plug-in for The Gimp program                         */
/* Copyright (C) 2001/02 Frank Lauterwald                                        */
/*===============================================================================*/
/* E-mail: frankl@gmx.net                                                        */
/* You can contact the original The Gimp authors at gimp@xcf.berkeley.edu        */
/*===============================================================================*/
/* This program is free software; you can redistribute it and/or modify it under */
/* the terms of the GNU General Public License as published by the Free Software */
/* Foundation; either version 2 of the License, or (at your option) any later    */
/* version.                                                                      */
/*===============================================================================*/
/* This program is distributed in the hope that it will be useful, but WITHOUT   */
/* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS */
/* FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.*/
/*===============================================================================*/
/* You should have received a copy of the GNU General Public License along with  */
/* this program; if not, write to the Free Software Foundation, Inc., 675 Mass   */
/* Ave, Cambridge, MA 02139, USA.                                                */
/*===============================================================================*/
/* In other words, you can't sue me for whatever happens while using this ;)     */
/*********************************************************************************/
/* CHANGELOG									 */
/* Date		Version		Comment						 */
/* 2001 Oct. 1,    0.30		first "release"					 */
/* 2001 Oct  4	   0.32		changed behaviour on pixels where the filter-	 */
/*				integral is zero. Formerly yellow (for debugging)*/
/*				now input color					 */
/* 2002 Jan 17	   x.xx (unusable)						 */
/*				began trying to include the old noise stuff into */
/*				my functions. As I cannot integrate over the	 */
/*				given noise functions (its probably possible	 */
/*				but I dont think it is worth it), this noise	 */
/*				thing is only possible with Euler		 */
/* 2002 Jan. 26,   0.50		release of first beta version			 */
/*********************************************************************************/

/*********************************************************************************/
/* This file is included by lic_algorithm.c					 */
/* It contains exactly two procedures that each compute a single output pixel	 */
/* One uses the algorithm described by cabral the other an euler-like approach   */
/* ---------------------------------------------------------------------------- */
/* it is included several times with different #defines set thus creating	 */
/*    different procedures doing very similar things				 */
/* Example: there are about 5 lines of code different depending on whether the 	 */
/*    used color model is rgb or grayscale.					 */
/*    We create two procedures nevertheless in order to optimize for speed as    */
/*    much as possible. This is done by including this file twice and 		 */
/*    and a #define colormodel rgb or #define colormodel gray or something	 */
/*    similar. The same thing applies to LI-Convolution versus DDA-Convolution   */
/*    and for blockfilter/other filterfunction					 */
/* ***************************************************************************** */

/* ***************************************************************************** */
/* There are, however, some helper functions					 */
/* ***************************************************************************** */
#ifndef __algorithm_include_c__
#define __algorithm_include_c__
inline void write_result_pixel_rgb(guint32 x, guint32 y, gpointer pixel)
{ guint32	index = (x+(y*(variables.outputblockwidth)))*3;
		variables.outputpixelregiondata[index++]=((rgbpixel*)pixel)->r; variables.outputpixelregiondata[index++]=((rgbpixel*)pixel)->g; variables.outputpixelregiondata[index]=((rgbpixel*)pixel)->b;
}

inline void write_result_pixel_gray(guint32 x, guint32 y, gpointer pixel)
{ guint32	index = (x+(y*(variables.outputblockwidth)))*1;
		variables.outputpixelregiondata[index]=((graypixel*)pixel)->b;
}
#endif


/* ***************************************************************************************************************************** */
/* Beginning of Euler function													 *(
/* ***************************************************************************************************************************** */
/* This is included only if:													 */
/* a) _noise_ is not defined or													 */
/* b) _filter_anyfilter_ is defined												 */
/* This leads to the following three cases being included:									 */
/* 1. convolve with image, blockfilter												 */
/* 2. convolve with image, anyfilter												 */
/* 3. convolve with noise, anyfilter												 */
/* ***************************************************************************************************************************** */
#ifndef _noise_ /* if _noise_ is defined, we try to simulate the 'noise' behaviour of Toms lic-filter */
	#ifdef _filter_blockfilter_
	#ifdef _algorithm_lic_
	   #ifdef _colormodel_rgb_
   		void algorithm_lic_euler_compute_pixel_rgb_blockfilter(gint block_x, gint block_y, gint global_x, gint global_y)
	   #endif
	   #ifdef _colormodel_rgba_
   		void algorithm_lic_euler_compute_pixel_rgba_blockfilter(gint block_x, gint block_y, gint global_x, gint global_y)
	   #endif
	   #ifdef _colormodel_gray_
	   	void algorithm_lic_euler_compute_pixel_gray_blockfilter(gint block_x, gint block_y, gint global_x, gint global_y)
	   #endif
	   #ifdef _colormodel_graya_
	   	void algorithm_lic_euler_compute_pixel_graya_blockfilter(gint block_x, gint block_y, gint global_x, gint global_y)
	   #endif
	#endif
	#ifdef _algorithm_dda_
   	  #ifdef _colormodel_rgb_
	   	void algorithm_dda_euler_compute_pixel_rgb_blockfilter(gint block_x, gint block_y, gint global_x, gint global_y)
	   #endif
	   #ifdef _colormodel_rgba_
	   	void algorithm_dda_euler_compute_pixel_rgba_blockfilter(gint block_x, gint block_y, gint global_x, gint global_y)
	   #endif
	   #ifdef _colormodel_gray_
	   	void algorithm_dda_euler_compute_pixel_gray_blockfilter(gint block_x, gint block_y, gint global_x, gint global_y)
	   #endif
	   #ifdef _colormodel_graya_
	   	void algorithm_dda_euler_compute_pixel_graya_blockfilter(gint block_x, gint block_y, gint global_x, gint global_y)
	   #endif
	#endif
	#endif /* _filter_blockfilter_ */
	#ifndef _filter_blockfilter_
	#ifdef _algorithm_lic_
	   #ifdef _colormodel_rgb_
	   	void algorithm_lic_euler_compute_pixel_rgb_anyfilter(gint block_x, gint block_y, gint global_x, gint global_y)
	   #endif
	   #ifdef _colormodel_rgba_
	   	void algorithm_lic_euler_compute_pixel_rgba_anyfilter(gint block_x, gint block_y, gint global_x, gint global_y)
	   #endif
	   #ifdef _colormodel_gray_
	   	void algorithm_lic_euler_compute_pixel_gray_anyfilter(gint block_x, gint block_y, gint global_x, gint global_y)
	   #endif
	   #ifdef _colormodel_graya_
	   	void algorithm_lic_euler_compute_pixel_graya_anyfilter(gint block_x, gint block_y, gint global_x, gint global_y)
	   #endif
	#endif /* _algorithm_lic_	*/
	#ifdef _algorithm_dda_
	   #ifdef _colormodel_rgb_
	   	void algorithm_dda_euler_compute_pixel_rgb_anyfilter(gint block_x, gint block_y, gint global_x, gint global_y)
	   #endif
	   #ifdef _colormodel_rgba_
	   	void algorithm_dda_euler_compute_pixel_rgba_anyfilter(gint block_x, gint block_y, gint global_x, gint global_y)
	   #endif
	   #ifdef _colormodel_gray_
	   	void algorithm_dda_euler_compute_pixel_gray_anyfilter(gint block_x, gint block_y, gint global_x, gint global_y)
	   #endif
	   #ifdef _colormodel_graya_
	   	void algorithm_dda_euler_compute_pixel_graya_anyfilter(gint block_x, gint block_y, gint global_x, gint global_y)
	   #endif
	#endif /* _algorithm_dda_   	   */
	#endif /* not _filter_blockfilter_ */
#endif /* not noise */
#ifdef _noise_
	#ifndef _filter_blockfilter_
	#ifdef _algorithm_lic_
	   #ifdef _colormodel_rgb_
	   	void algorithm_lic_euler_compute_pixel_rgb_anyfilter_noise(gint block_x, gint block_y, gint global_x, gint global_y)
	   #endif
	   #ifdef _colormodel_rgba_
	   	void algorithm_lic_euler_compute_pixel_rgba_anyfilter_noise(gint block_x, gint block_y, gint global_x, gint global_y)
	   #endif
	   #ifdef _colormodel_gray_
	   	void algorithm_lic_euler_compute_pixel_gray_anyfilter_noise(gint block_x, gint block_y, gint global_x, gint global_y)
	   #endif
	   #ifdef _colormodel_graya_
	   	void algorithm_lic_euler_compute_pixel_graya_anyfilter_noise(gint block_x, gint block_y, gint global_x, gint global_y)
	   #endif
	#endif /* _algorithm_lic_	*/
	#ifdef _algorithm_dda_
	   #ifdef _colormodel_rgb_
	   	void algorithm_dda_euler_compute_pixel_rgb_anyfilter_noise(gint block_x, gint block_y, gint global_x, gint global_y)
	   #endif
	   #ifdef _colormodel_rgba_
	   	void algorithm_dda_euler_compute_pixel_rgba_anyfilter_noise(gint block_x, gint block_y, gint global_x, gint global_y)
	   #endif
	   #ifdef _colormodel_gray_
	   	void algorithm_dda_euler_compute_pixel_gray_anyfilter_noise(gint block_x, gint block_y, gint global_x, gint global_y)
	   #endif
	   #ifdef _colormodel_graya_
	   	void algorithm_dda_euler_compute_pixel_graya_anyfilter_noise(gint block_x, gint block_y, gint global_x, gint global_y)
	   #endif
	#endif /* _algorithm_dda_   	   */
	#endif /* not _filter_blockfilter_ */
#endif /* _noise */
{	/* variables	*/
	gdouble pos;
	gfloat  normalize_sum;
	gfloat  filter_value;
#ifdef _noise_
	gfloat  noise_sum;
#endif
#ifdef _colormodel_rgb_
	rgbpixel      *currentpixel;		  /* the texel that is just being processed		*/
	rgbfloatpixel pix  = RGBFLOATPIXEL_BLACK; /* will hold the so-far calculated sum for output	*/
#endif
#ifdef _colormodel_gray_
	graypixel       *currentpixel;
	grayfloatpixel pix = GRAYFLOATPIXEL_BLACK;
#endif
	guint16 vector_index;
	gfloat currentx, currenty;
	guchar buf[4]; 				  /* buffer for writing the pixel	*/
	gint32 index;
	float addx, addy; /* how much will be added to the current position for each step */


	gint32 cache_globalx_minus_blockx;
	gint32 cache_globaly_minus_blocky;
	gint32 cache_delta_x, cache_delta_y;		/* TODO: describe why this one is needed	*/
	float cache_stepsize;


	/* initialization */
	normalize_sum=0;
#ifdef _noise_
	noise_sum = 0.0;
#endif


	/* forward loop	  */
	pos=0;
	currentx=global_x+0.5; currenty=global_y+0.5;
	cache_globalx_minus_blockx=global_x-block_x;
	cache_globaly_minus_blocky=global_y-block_y;
	cache_stepsize=parameters.algorithm.extras.ddaEuler.stepsize;
	cache_delta_x =-variables.input_swappedin_x1+variables.output_swappedin_x1; /* the difference between swapin_output_x1 and swapin_input_x1 */
	cache_delta_y =-variables.input_swappedin_y1+variables.output_swappedin_y1;

	/* these +1s come from the fact that the vector field has a one-pixel thick border around the real vectors */
	vector_index=variables.vectors->data[(gint32)(floor(currentx+1)+floor(currenty+1)*(variables.width+2))];
#ifdef _algorithm_dda_
	addx = variables.floatvectors.field[vector_index].x*cache_stepsize;
	addy = variables.floatvectors.field[vector_index].y*cache_stepsize;
#endif
	if (IS_NULLVECTOR(vector_index))
	{
#ifdef _colormodel_rgb_
		index = (((gint32)currentx-cache_globalx_minus_blockx+cache_delta_x)+((gint32)currenty-cache_globaly_minus_blocky+cache_delta_y)*variables.inputblockwidth)*3;
		currentpixel=(rgbpixel*)(&(variables.inputpixelregiondata[index]));
		write_result_pixel_rgb(block_x, block_y, currentpixel);
#endif
#ifdef _colormodel_gray_
		index = (((gint32)currentx-cache_globalx_minus_blockx+cache_delta_x)+((gint32)currenty-cache_globaly_minus_blocky+cache_delta_y)*variables.inputblockwidth)*1;
		currentpixel=(graypixel*)(&(variables.inputpixelregiondata[index]));
		write_result_pixel_gray(block_x, block_y, currentpixel);
#endif
		return;
	}


	/* MAIN LOOP (FORWARD) */
	while (pos<parameters.filterlength)
	{

#ifdef _algorithm_lic_
			vector_index=variables.vectors->data[(gint32)((floor(currentx+1)+(floor(currenty+1)*(variables.width+2))))];
			if (IS_NULLVECTOR(vector_index)) { break; }
#endif
#ifdef _algorithm_dda_
			if ( (((gint32)(currentx))==0) || (((gint32)(currenty))==0) || (((gint32)(currentx))==variables.width+1) || (((gint32)(currenty))==variables.width+1))
			{ break; }
#endif

#ifndef _filter_blockfilter_
			filter_value   = variables.filter(pos);
			normalize_sum += filter_value;
#ifdef _noise_
		// simulate old-lic behaviour
			//fprintf(stderr, "sum before %f ", noise_sum);
			//fprintf(stderr, "%i %i \n", block_x, block_y);
			noise_sum+=filter_value*oldlic_noise(currentx, currenty);
			//fprintf(stderr, "%f %f sum now %f \n", filter_value, oldlic_noise(currentx, currenty), noise_sum);
#endif
#endif
#ifdef _filter_blockfilter_
			normalize_sum+=1;
#endif

#ifdef _colormodel_rgb_
			index = (((gint32)currentx-cache_globalx_minus_blockx+cache_delta_x)+((gint32)currenty-cache_globaly_minus_blocky+cache_delta_y)*variables.inputblockwidth)*3;
			currentpixel=(rgbpixel*)(&(variables.inputpixelregiondata[index]));
#ifndef _filter_blockfilter_
				pix.r+=(gfloat)currentpixel->r*filter_value; pix.g+=(gfloat)currentpixel->g*filter_value; pix.b+=(gfloat)currentpixel->b*filter_value;
#endif
#ifdef  _filter_blockfilter_
				pix.r+=(gfloat)currentpixel->r; pix.g+=(gfloat)currentpixel->g; pix.b+=(gfloat)currentpixel->b;
#endif
#endif
#ifdef _colormodel_gray_
			index = (((gint32)currentx-cache_globalx_minus_blockx+cache_delta_x)+((gint32)currenty-cache_globaly_minus_blocky+cache_delta_y)*variables.inputblockwidth)*1;
			currentpixel=(graypixel*)(&(variables.inputpixelregiondata[index]));
#ifndef _filter_blockfilter_
				pix.b+=(gfloat)currentpixel->b*filter_value;
#endif
#ifdef  _filter_blockfilter_
				pix.b+=(gfloat)currentpixel->b;
#endif
#endif

		pos+=cache_stepsize;

#ifdef _algorithm_lic_
			addx = variables.floatvectors.field[vector_index].x*cache_stepsize;
			addy = variables.floatvectors.field[vector_index].y*cache_stepsize;
#endif
		/* if algorithm is _algorithm_dda_ nothing is done: the old addx/addy values are retained */

		currentx+=addx; currenty+=addy;
	}

	/* backward loop */
	pos=0;
	currentx=global_x+0.5; currenty=global_y+0.5;
	cache_globalx_minus_blockx=global_x-block_x;
	cache_globaly_minus_blocky=global_y-block_y;
	cache_stepsize=parameters.algorithm.extras.ddaEuler.stepsize;
	cache_delta_x =-variables.input_swappedin_x1+variables.output_swappedin_x1;/* the difference between swapin_output_x1 and swapin_input_x1 */
	cache_delta_y =-variables.input_swappedin_y1+variables.output_swappedin_y1;

	/* these +1s come from the fact that the vector field has a one-pixel thick border around the real vectors */
	vector_index=variables.vectors->data[(gint32)(floor(currentx+1)+floor(currenty+1)*(variables.width+2))];
#ifdef _algorithm_dda_
	addx = variables.floatvectors.field[vector_index].x*cache_stepsize;
	addy = variables.floatvectors.field[vector_index].y*cache_stepsize;
#endif
	if (IS_NULLVECTOR(vector_index))
	{	TOUT("Nullvektor");
		return;
	}


	/* MAIN LOOP (BACKWARD)																		*/
	/* the backward loop differs from the forward one at three places:1) pos+= -> pos-= 2) currentx+=addx -> currentx-=addx 3) currenty+=addx -> currenty-=addx	*/
	while (pos>-parameters.filterlength)
	{
#ifdef _algorithm_lic_
			vector_index=variables.vectors->data[(gint32)((floor(currentx+1)+(floor(currenty+1)*(variables.width+2))))];
			if (IS_NULLVECTOR(vector_index)) { break; }
#endif
#ifdef _algorithm_dda_
			if ((((gint32)(currentx))==0) || (((gint32)(currenty))==0) || (((gint32)(currentx))==variables.width+1) || (((gint32)(currenty))==variables.width+1))
			{ break; }
#endif

#ifndef _filter_blockfilter_
			filter_value   = variables.filter(pos);
			normalize_sum += filter_value;
#ifdef _noise_
		// simulate old-lic behaviour
			noise_sum+=filter_value*oldlic_noise(currentx, currenty);
		//	fprintf(stderr, "%f %f sum now %f \n", filter_value, oldlic_noise(currentx, currenty), noise_sum);
#endif
#endif
#ifdef _filter_blockfilter_
			normalize_sum+=1;
#endif


		/* this is the colormodel-specific part */
#ifdef _colormodel_rgb_
			index = (((gint32)currentx-cache_globalx_minus_blockx+cache_delta_x)+((gint32)currenty-cache_globaly_minus_blocky+cache_delta_y)*variables.inputblockwidth)*3;
			currentpixel=(rgbpixel*)(&(variables.inputpixelregiondata[index]));
#ifndef _filter_blockfilter_
			pix.r+=(gfloat)currentpixel->r*filter_value; pix.g+=(gfloat)currentpixel->g*filter_value; pix.b+=(gfloat)currentpixel->b*filter_value;
#endif
#ifdef  _filter_blockfilter_
			pix.r+=(gfloat)currentpixel->r; pix.g+=(gfloat)currentpixel->g; pix.b+=(gfloat)currentpixel->b;
#endif
#endif
#ifdef _colormodel_gray_
	index = (((gint32)currentx-cache_globalx_minus_blockx+cache_delta_x)+((gint32)currenty-cache_globaly_minus_blocky+cache_delta_y)*variables.inputblockwidth)*1;
	currentpixel=(graypixel*)(&(variables.inputpixelregiondata[index]));
#ifndef _filter_blockfilter_
		pix.b+=(gfloat)currentpixel->b*filter_value;
#endif
#ifdef  _filter_blockfilter_
		pix.b+=(gfloat)currentpixel->b;
#endif
#endif

		pos-=cache_stepsize;

#ifdef _algorithm_lic_
			addx = variables.floatvectors.field[vector_index].x*cache_stepsize;
			addy = variables.floatvectors.field[vector_index].y*cache_stepsize;
#endif
		/* if algorithm is _algorithm_dda_ nothing is done: the old addx/addy values are retained */

		currentx-=addx; currenty-=addy;
	}


	/* write resulting pixel */
#ifndef _noise_
	#ifdef _colormodel_rgb_
		if(normalize_sum>0)
		{ buf[0]=pix.r/(normalize_sum); buf[1]=pix.g/(normalize_sum); buf[2]=pix.b/(normalize_sum); /*buf[3]=255;*/
		  write_result_pixel_rgb(block_x, block_y, (gpointer)&buf);
		}
		else
		{ index = (((gint32)currentx-cache_globalx_minus_blockx+cache_delta_x)+((gint32)currenty-cache_globaly_minus_blocky+cache_delta_y)*variables.inputblockwidth)*3;
		  currentpixel=(rgbpixel*)(&(variables.inputpixelregiondata[index]));
		  write_result_pixel_rgb(block_x, block_y, (gpointer)currentpixel);
		}

	#endif
	#ifdef _colormodel_gray_
		if(normalize_sum>0)
		{ buf[0]=pix.b/(normalize_sum);
		  write_result_pixel_gray(block_x, block_y, (gpointer)&buf);
		}
		else
		{ index = (((gint32)currentx-cache_globalx_minus_blockx+cache_delta_x)+((gint32)currenty-cache_globaly_minus_blocky+cache_delta_y)*variables.inputblockwidth)*1;
		  currentpixel=(graypixel*)(&(variables.inputpixelregiondata[index]));
		  write_result_pixel_gray(block_x, block_y, (gpointer)currentpixel);
		}

	#endif
#endif /* not noise */
#ifdef _noise_ /* this is the main between the normal and the simulated noise behaviour 	*/
	/* difference between normal and noise convolution: 				*/
	/* normal: result(x,y) = integral over a,b (input(a,b)*filter(...))		*/
	/* noise:  result(x,y) = input(x,y) * integral over a,b (noise(a,b)*filter(...))*/
	#ifdef _colormodel_rgb_
		if(normalize_sum>0)
		{ gfloat factor;
		  /* get input texture at given position */
		  index = (((gint32)currentx-cache_globalx_minus_blockx+cache_delta_x)+((gint32)currenty-cache_globaly_minus_blocky+cache_delta_y)*variables.inputblockwidth)*3;
		  currentpixel=(rgbpixel*)(&(variables.inputpixelregiondata[index]));

		  factor = noise_sum*2;//normalize_sum;
		  fprintf(stderr,"Summe __ : %f \n",factor);
	factor=(factor-parameters.noiseoptions.minv)/(parameters.noiseoptions.maxv-parameters.noiseoptions.minv);



	if (factor<0.0)
	   factor=0.0;

	if (factor>1.0)
	  factor=1.0;

	factor=(factor/2.0)+0.5;
		  //fprintf(stderr, "%i %i \n", global_x, global_y);
		  //fprintf(stderr, "%f %f %f \n",noise_sum, normalize_sum, factor);
		  buf[0]=currentpixel->r*factor; buf[1]=currentpixel->g*factor; buf[2]=currentpixel->b*factor;
		  write_result_pixel_rgb(block_x, block_y, (gpointer)&buf);
		  //fprintf(stderr,"Noise at pos: %f ",oldlic_noise(1.600872, -0.077003));
		}
		else
		{ index = (((gint32)currentx-cache_globalx_minus_blockx+cache_delta_x)+((gint32)currenty-cache_globaly_minus_blocky+cache_delta_y)*variables.inputblockwidth)*3;
		  currentpixel=(rgbpixel*)(&(variables.inputpixelregiondata[index]));
		  TOUT("noise: normalize_sum=0. return input or black? currently input ");
		  write_result_pixel_rgb(block_x, block_y, (gpointer)currentpixel);
		}

	#endif
	#ifdef _colormodel_gray_
		if(normalize_sum>0)
		{ gfloat factor;
		  factor = noise_sum/normalize_sum;
		  index = (((gint32)currentx-cache_globalx_minus_blockx+cache_delta_x)+((gint32)currenty-cache_globaly_minus_blocky+cache_delta_y)*variables.inputblockwidth)*1;
		  currentpixel=(graypixel*)(&(variables.inputpixelregiondata[index]));
		  buf[0]=currentpixel->b*factor;
		  write_result_pixel_gray(block_x, block_y, (gpointer)&buf);
		}
		else
		{ index = (((gint32)currentx-cache_globalx_minus_blockx+cache_delta_x)+((gint32)currenty-cache_globaly_minus_blocky+cache_delta_y)*variables.inputblockwidth)*1;
		  currentpixel=(graypixel*)(&(variables.inputpixelregiondata[index]));
		  TOUT("noise: normalize_sum=0.return input or black? currently input ");
		  write_result_pixel_gray(block_x, block_y, (gpointer)currentpixel);
		}

	#endif
#endif /* noise */


}
/* ***************************************************************************************************************************** */
/* End of euler function													 *(
/* ***************************************************************************************************************************** */



/* ***************************************************************************************************************************** */
/* Beginning of Cabral function													 *(
/* ***************************************************************************************************************************** */
/* Function is only included when _noise_ is not defined									 */
/* ***************************************************************************************************************************** */
#ifndef _noise_

#ifdef _filter_blockfilter_
#ifdef _algorithm_lic_
   #ifdef _colormodel_rgb_
   	void algorithm_lic_cabral_compute_pixel_rgb_blockfilter(gint block_x, gint block_y, gint global_x, gint global_y)
   #endif
   #ifdef _colormodel_rgba_
   	void algorithm_lic_cabral_compute_pixel_rgba_blockfilter(gint block_x, gint block_y, gint global_x, gint global_y)
   #endif
   #ifdef _colormodel_gray_
   	void algorithm_lic_cabral_compute_pixel_gray_blockfilter(gint block_x, gint block_y, gint global_x, gint global_y)
   #endif
   #ifdef _colormodel_graya_
   	void algorithm_lic_cabral_compute_pixel_graya_blockfilter(gint block_x, gint block_y, gint global_x, gint global_y)
   #endif
#endif
#ifdef _algorithm_dda_
   #ifdef _colormodel_rgb_
   	void algorithm_dda_cabral_compute_pixel_rgb_blockfilter(gint block_x, gint block_y, gint global_x, gint global_y)
   #endif
   #ifdef _colormodel_rgba_
   	void algorithm_dda_cabral_compute_pixel_rgba_blockfilter(gint block_x, gint block_y, gint global_x, gint global_y)
   #endif
   #ifdef _colormodel_gray_
   	void algorithm_dda_cabral_compute_pixel_gray_blockfilter(gint block_x, gint block_y, gint global_x, gint global_y)
   #endif
   #ifdef _colormodel_graya_
   	void algorithm_dda_cabral_compute_pixel_graya_blockfilter(gint block_x, gint block_y, gint global_x, gint global_y)
   #endif
#endif /* _algorithm_dda_ 	*/
#endif /* _filter_blockfilter_ 	*/
#ifndef _filter_blockfilter_
#ifdef _algorithm_lic_
   #ifdef _colormodel_rgb_
   	void algorithm_lic_cabral_compute_pixel_rgb_anyfilter(gint block_x, gint block_y, gint global_x, gint global_y)
   #endif
   #ifdef _colormodel_rgba_
   	void algorithm_lic_cabral_compute_pixel_rgba_anyfilter(gint block_x, gint block_y, gint global_x, gint global_y)
   #endif
   #ifdef _colormodel_gray_
   	void algorithm_lic_cabral_compute_pixel_gray_anyfilter(gint block_x, gint block_y, gint global_x, gint global_y)
   #endif
   #ifdef _colormodel_graya_
   	void algorithm_lic_cabral_compute_pixel_graya_anyfilter(gint block_x, gint block_y, gint global_x, gint global_y)
   #endif
#endif /* _algorithm_lic_	*/
#ifdef _algorithm_dda_
   #ifdef _colormodel_rgb_
   	void algorithm_dda_cabral_compute_pixel_rgb_anyfilter(gint block_x, gint block_y, gint global_x, gint global_y)
   #endif
   #ifdef _colormodel_rgba_
   	void algorithm_dda_cabral_compute_pixel_rgba_anyfilter(gint block_x, gint block_y, gint global_x, gint global_y)
   #endif
   #ifdef _colormodel_gray_
   	void algorithm_dda_cabral_compute_pixel_gray_anyfilter(gint block_x, gint block_y, gint global_x, gint global_y)
   #endif
   #ifdef _colormodel_graya_
   	void algorithm_dda_cabral_compute_pixel_graya_anyfilter(gint block_x, gint block_y, gint global_x, gint global_y)
   #endif
#endif /* _algorithm_dda_   	   */
#endif /* not _filter_blockfilter: */
{	/* variables	*/
	gdouble pos;
	float  normalize_sum;
	float  filter_integral;
#ifdef _colormodel_rgb_
	rgbpixel      *currentpixel;		  /* the texel that is just being processed		*/
	rgbfloatpixel pix  = RGBFLOATPIXEL_BLACK; /* will hold the so-far calculated sum for output	*/
#endif
#ifdef _colormodel_gray_
	graypixel       *currentpixel;
	grayfloatpixel pix = GRAYFLOATPIXEL_BLACK;
#endif
	guint16 vector_index, oldvector_index;
	float currentx, currenty;
	float currentix, currentiy;		  /* those are currentx and currenty _rounded down_ */
	guchar buf[4]; 				  /* buffer for writing the pixel	*/
	gint32 index;
	float addx, addy; 	/* how much will be added to the current position		*/
	float max, maxx, maxy; 	/* now many times we may move the current vector forward until we leave the current pixel */
	floatvector v; 		/* the current floatvector as well as the last one     				  */


	gint32 cache_globalx_minus_blockx;
	gint32 cache_globaly_minus_blocky;
	gint32 cache_delta_x, cache_delta_y;		/* TODO: describe why this one is needed	*/
	float cache_stepsize;


	/* initialization */
	normalize_sum=0;

	/* forward loop	  */
	pos=0;
	currentx=global_x+0.5; currenty=global_y+0.5;
	cache_globalx_minus_blockx=global_x-block_x;
	cache_globaly_minus_blocky=global_y-block_y;
	cache_delta_x =-variables.input_swappedin_x1+variables.output_swappedin_x1; /* the difference between swapin_output_x1 and swapin_input_x1 */
	cache_delta_y =-variables.input_swappedin_y1+variables.output_swappedin_y1;
	oldvector_index = NULLVECTOR;

	/* these +1s come from the fact that the vector field has a one-pixel thick border around the real vectors */
	vector_index=variables.vectors->data[(gint32)(floor(currentx+1)+floor(currenty+1)*(variables.width+2))];
	if (IS_NULLVECTOR(vector_index))
	{

	#ifdef _colormodel_rgb_
		index = (((gint32)currentx-cache_globalx_minus_blockx+cache_delta_x)+((gint32)currenty-cache_globaly_minus_blocky+cache_delta_y)*variables.inputblockwidth)*3;
		currentpixel=(rgbpixel*)(&(variables.inputpixelregiondata[index]));
		write_result_pixel_rgb(block_x, block_y, currentpixel);
		  buf[0]=0; buf[1]=255; buf[2]=255; /*buf[3]=255;*/
		  write_result_pixel_rgb(block_x, block_y, (gpointer)&buf);
	#endif
	#ifdef _colormodel_gray_
		index = (((gint32)currentx-cache_globalx_minus_blockx+cache_delta_x)+((gint32)currenty-cache_globaly_minus_blocky+cache_delta_y)*variables.inputblockwidth)*1;
		currentpixel=(graypixel*)(&(variables.inputpixelregiondata[index]));
		write_result_pixel_gray(block_x, block_y, currentpixel);
	#endif
		return;
	}

	/* MAIN LOOP (FORWARD) */
	while (pos<parameters.filterlength)
	{

	#ifdef _algorithm_lic_
			vector_index=variables.vectors->data[(gint32)((floor(currentx+1)+(floor(currenty+1)*(variables.width+2))))];
			if (IS_NULLVECTOR(vector_index)) { break; }
	#endif
	#ifdef _algorithm_dda_
			if ( (((gint32)(currentx))==0) || (((gint32)(currenty))==0) || (((gint32)(currentx))==variables.width+1) || (((gint32)(currenty))==variables.width+1))
			{ break; }
	#endif

	 /*terminate if current vector is exact opposite of last one */
	if ((oldvector_index==(guint16)(vector_index+32768)) || (oldvector_index==(guint16)(vector_index+32767)) && (!IS_NULLVECTOR(oldvector_index)))
	{ break; }


	/* get vector */
	v = variables.floatvectors.field[vector_index];
	/* determine stepsize */
	currentix = floor(currentx); currentiy = floor(currenty);
	maxx = INFINITY;
	maxy = INFINITY;
	if (v.x>0)
		{ maxx = (currentx-currentix)/v.x; }
	else { if ( v.x<0)
          	{ maxx = (currentx-(currentix+1))/v.x; } }
	if (v.y>0)
	  	{ maxy = (currenty-currentiy)/v.y;  }
	else   	{ if (v.y<0)
		{ maxy = (currenty-(currentiy+1))/v.y; } }
	if (maxx >= maxy)
		{ max = maxy+SMALLNUMBER; }
	else   	{ max = maxx+SMALLNUMBER; }
	if ((max<MINSTEPSIZE) || (max==INFINITY))
	{ break; }
	/* -- determine stepsize */

	/* determine filterintegral */
	#ifndef _filter_blockfilter_
			filter_integral   = variables.filter_integrate(pos,pos+max);
	#endif
	#ifdef _filter_blockfilter_
			filter_integral   = max;
	#endif
	normalize_sum += filter_integral;
	/* -- determine filterintegral */

	#ifdef _colormodel_rgb_
			index = (((gint32)currentx-cache_globalx_minus_blockx+cache_delta_x)+((gint32)currenty-cache_globaly_minus_blocky+cache_delta_y)*variables.inputblockwidth)*3;
			currentpixel=(rgbpixel*)(&(variables.inputpixelregiondata[index]));
			pix.r+=((gfloat)currentpixel->r)*filter_integral; pix.g+=((gfloat)currentpixel->g)*filter_integral; pix.b+=((gfloat)currentpixel->b)*filter_integral;
			/*fprintf(stderr, "index: %i, vector:(%f / %f) \n",index,v.x,v.y);*/
	#endif
	#ifdef _colormodel_gray_
			index = (((gint32)currentx-cache_globalx_minus_blockx+cache_delta_x)+((gint32)currenty-cache_globaly_minus_blocky+cache_delta_y)*variables.inputblockwidth)*1;
			currentpixel=(graypixel*)(&(variables.inputpixelregiondata[index]));
			pix.b+=(gfloat)currentpixel->b*filter_integral;
	#endif

	pos+=max;

	addx = max*v.x;
	addy = max*v.y;

	currentx-=addx; currenty-=addy;
	/*fprintf(stderr, "new position: %f / %f \n", currentx, currenty);*/
	oldvector_index = vector_index;
	}

	/* backward loop */
	pos=0;
	currentx=global_x+0.5; currenty=global_y+0.5;
	cache_globalx_minus_blockx=global_x-block_x;
	cache_globaly_minus_blocky=global_y-block_y;
	cache_delta_x =-variables.input_swappedin_x1+variables.output_swappedin_x1; /* the difference between swapin_output_x1 and swapin_input_x1 */
	cache_delta_y =-variables.input_swappedin_y1+variables.output_swappedin_y1;
	oldvector_index = NULLVECTOR;

	/* these +1s come from the fact that the vector field has a one-pixel thick border around the real vectors */
	vector_index=variables.vectors->data[(gint32)(floor(currentx+1)+floor(currenty+1)*(variables.width+2))];
	if (IS_NULLVECTOR(vector_index))
	{

	#ifdef _colormodel_rgb_
		index = (((gint32)currentx-cache_globalx_minus_blockx+cache_delta_x)+((gint32)currenty-cache_globaly_minus_blocky+cache_delta_y)*variables.inputblockwidth)*3;
		currentpixel=(rgbpixel*)(&(variables.inputpixelregiondata[index]));
		write_result_pixel_rgb(block_x, block_y, currentpixel);
	#endif
	#ifdef _colormodel_gray_
		index = (((gint32)currentx-cache_globalx_minus_blockx+cache_delta_x)+((gint32)currenty-cache_globaly_minus_blocky+cache_delta_y)*variables.inputblockwidth)*1;
		currentpixel=(graypixel*)(&(variables.inputpixelregiondata[index]));
		write_result_pixel_gray(block_x, block_y, currentpixel);
	#endif
		return;
	}

	/* MAIN LOOP (FORWARD) */
	while (pos<parameters.filterlength)
	{

	#ifdef _algorithm_lic_
			vector_index=variables.vectors->data[(gint32)((floor(currentx+1)+(floor(currenty+1)*(variables.width+2))))];
			if (IS_NULLVECTOR(vector_index)) { break; }
	#endif
	#ifdef _algorithm_dda_
			if ( (((gint32)(currentx))==0) || (((gint32)(currenty))==0) || (((gint32)(currentx))==variables.width+1) || (((gint32)(currenty))==variables.width+1))
			{ break; }
	#endif

	/* terminate if current vector is exact opposite of last one */
	if ((oldvector_index==(guint16)(vector_index+32768)) || (oldvector_index==(guint16)(vector_index+32767)) && (!IS_NULLVECTOR(oldvector_index)))
	{ break; }


	/* get vector */
	v = variables.floatvectors.field[vector_index];
	/* determine stepsize */
	currentix = floor(currentx); currentiy = floor(currenty);
	maxx = INFINITY;
	maxy = INFINITY;
	if (v.x>0)
		{ maxx = -(currentx-currentix)/v.x; }
	else { if ( v.x<0)
          	{ maxx = -(currentx-(currentix+1))/v.x; } }
	if (v.y>0)
	  	{ maxy = -(currenty-currentiy)/v.y;  }
	else   	{ if (v.y<0)
		{ maxy = -(currenty-(currentiy+1))/v.y; } }
	if (maxx >= maxy)
		{ max = maxy+SMALLNUMBER; }
	else   	{ max = maxx+SMALLNUMBER; }
	if ((max<MINSTEPSIZE) || (max==INFINITY))
	{ break; }
	/* -- determine stepsize */

	/* determine filterintegral */
	/* -- determine filterintegral */

	#ifndef _filter_blockfilter_
			filter_integral   = variables.filter_integrate(pos,pos+max);
	#endif
	#ifdef _filter_blockfilter_
			filter_integral   = max;
	#endif
	normalize_sum += filter_integral;

	#ifdef _colormodel_rgb_
			index = (((gint32)currentx-cache_globalx_minus_blockx+cache_delta_x)+((gint32)currenty-cache_globaly_minus_blocky+cache_delta_y)*variables.inputblockwidth)*3;
			currentpixel=(rgbpixel*)(&(variables.inputpixelregiondata[index]));
			pix.r+=((gfloat)currentpixel->r)*filter_integral; pix.g+=((gfloat)currentpixel->g)*filter_integral; pix.b+=((gfloat)currentpixel->b)*filter_integral;
	#endif
	#ifdef _colormodel_gray_
			index = (((gint32)currentx-cache_globalx_minus_blockx+cache_delta_x)+((gint32)currenty-cache_globaly_minus_blocky+cache_delta_y)*variables.inputblockwidth)*1;
			currentpixel=(graypixel*)(&(variables.inputpixelregiondata[index]));
			pix.b+=(gfloat)currentpixel->b*filter_integral;
	#endif

	pos-=max;

	addx = max*v.x;
	addy = max*v.y;

	currentx-=addx; currenty-=addy;
	oldvector_index = vector_index;
	}




	/* write resulting pixel */ /* This is exactly as in the euler function */
#ifdef _colormodel_rgb_
		if(normalize_sum>0)
		{ buf[0]=pix.r/(normalize_sum); buf[1]=pix.g/(normalize_sum); buf[2]=pix.b/(normalize_sum); /*buf[3]=255;*/
		  write_result_pixel_rgb(block_x, block_y, (gpointer)&buf);
		}
		else
		{ index = (((gint32)currentx-cache_globalx_minus_blockx+cache_delta_x)+((gint32)currenty-cache_globaly_minus_blocky+cache_delta_y)*variables.inputblockwidth)*3;
		  currentpixel=(rgbpixel*)(&(variables.inputpixelregiondata[index]));
		  write_result_pixel_rgb(block_x, block_y, (gpointer)currentpixel);
		  buf[0]=255; buf[1]=255; buf[2]=0; /*buf[3]=255;*/
		  write_result_pixel_rgb(block_x, block_y, (gpointer)&buf);
		}

#endif
#ifdef _colormodel_gray_
		if(normalize_sum>0)
		{ buf[0]=pix.b/(normalize_sum);
		  write_result_pixel_gray(block_x, block_y, (gpointer)&buf);
		}
		else
		{ index = (((gint32)currentx-cache_globalx_minus_blockx+cache_delta_x)+((gint32)currenty-cache_globaly_minus_blocky+cache_delta_y)*variables.inputblockwidth)*1;
		  currentpixel=(graypixel*)(&(variables.inputpixelregiondata[index]));
		  write_result_pixel_gray(block_x, block_y, (gpointer)currentpixel);
		}

#endif

}
#endif /* ifndef _noise_ */
/* ***************************************************************************************************************************** */
/* Ende of Cabral function													 *(
/* ***************************************************************************************************************************** */
