/*********************************************************************************/
/* LIC 0.50 -- image filter plug-in for The Gimp program                         */
/* Copyright (C) 2001/02 Frank Lauterwald                                        */
/*===============================================================================*/
/* E-mail: frankl@gmx.net                                                        */
/* You can contact the original The Gimp authors at gimp@xcf.berkeley.edu        */
/*===============================================================================*/
/* This program is free software; you can redistribute it and/or modify it under */
/* the terms of the GNU General Public License as published by the Free Software */
/* Foundation; either version 2 of the License, or (at your option) any later    */
/* version.                                                                      */
/*===============================================================================*/
/* This program is distributed in the hope that it will be useful, but WITHOUT   */
/* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS */
/* FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.*/
/*===============================================================================*/
/* You should have received a copy of the GNU General Public License along with  */
/* this program; if not, write to the Free Software Foundation, Inc., 675 Mass   */
/* Ave, Cambridge, MA 02139, USA.                                                */
/*===============================================================================*/
/* In other words, you can't sue me for whatever happens while using this ;)     */
/*********************************************************************************/
/* CHANGELOG									 */
/* Date		Version		Comment						 */
/* 2001 Oct. 1,    0.30		first "release"					 */
/* 2002 Jan. 26,   0.50		release of first beta version			 */
/*										 */
/*********************************************************************************/
/* This block is completely stolen from the original LIC-Filter			 */
/*********************************************************************************/
/*********************************************************************************/


#include "lic_noise.h"
#include "lic_globals.h"
#include "lic_vectors.h"
#include <libgimp/gimp.h>
#include <stdio.h>

gdouble noise(gdouble x,gdouble y);

// this is a pointer to an array of "random vectors"
guint16 *randomVectors;

/* ********************************************************************** */
/* First the three main functions of the old-lic-noise emulation	          */
/* ********************************************************************** */
/* Initialize everything for the further use of the noise emulation	  */
/* ********************************************************************** */
void oldlic_noise_init()  {
	int i,j;
	/* generate pseudo-random-vectors */
	if (!(randomVectors=malloc(parameters.noiseoptions.numx * parameters.noiseoptions.numy * sizeof(guint16)))) {
		fprintf(stderr, "oldlic_noise_init: cannot allocate memory \n"); exit(EXIT_FAILURE);
	}
	for (i=0; i < parameters.noiseoptions.numx; i++) {
		for (j=0; j < parameters.noiseoptions.numy; j++) {
			randomVectors[i + j*parameters.noiseoptions.numx] = rand()%65535+1;
		}
	}
	/* -- generate pseudo-random-vectors */
}
/* ********************************************************************** */


/* ********************************************************************** */
/* get the noise value at a given position				  */
/* ********************************************************************** */
gfloat oldlic_noise(gfloat x, gfloat y) {
//	return 0.0;
	gdouble i = noise(x,y);
	//fprintf(stderr, "noise1: %f \n",i);

  	/*i=(i-parameters.noiseoptions.minv)/(parameters.noiseoptions.maxv-parameters.noiseoptions.minv);

	if (i<0.0)
	   i=0.0;

	if (i>1.0)
	  i=1.0;

	i=(i/2.0)+0.5;*/

	//fprintf(stderr, "noise2: %f \n",i);
	return(i);
}
/* ********************************************************************** */

/* ********************************************************************** */
/* This is called to free up memory after using noise stuff		  */
/* ********************************************************************** */
void oldlic_noise_done() {
	free (randomVectors);
}
/* ********************************************************************** */



/* ********************************************************************** */
/* Now come a few helpers						  */
/* This is taken from Toms lic-plugin					  */
/* ********************************************************************** */

/************************************/
/* A nice 2nd order cubic spline :) */
/************************************/
inline gdouble cubic(gdouble t)
{
  gdouble at=fabs(t);

  if (at<1.0)
    return(2.0*at*at*at-3.0*at*at+1.0);

  return(0.0);
}

inline gdouble omega(gdouble u,gdouble v,gint i,gint j)
{

  while (i<0)
    i+=parameters.noiseoptions.numx;

  while (j<0)
    j+=parameters.noiseoptions.numy;

  i%=parameters.noiseoptions.numx;
  j%=parameters.noiseoptions.numy;
  return(cubic(u)*cubic(v)*(variables.floatvectors.field[randomVectors[i+j*parameters.noiseoptions.numx]].x*u+
  			    variables.floatvectors.field[randomVectors[i+j*parameters.noiseoptions.numx]].y*v));
}


/*************************************************************/
/* The noise function (2D variant of Perlins noise function) */
/*************************************************************/

gdouble noise(gdouble x,gdouble y)
{
  gint i,sti=(gint)floor(x/parameters.noiseoptions.dx);
  gint j,stj=(gint)floor(y/parameters.noiseoptions.dy);

  gdouble sum=0.0;

  /* Calculate the gdouble sum */
  /* ======================== */

  for (i=sti; i<=sti+1; i++)
    {
      for (j=stj; j<=stj+1; j++)
        sum+=omega((x-(gdouble)i*parameters.noiseoptions.dx)/parameters.noiseoptions.dx,(y-(gdouble)j*parameters.noiseoptions.dy)/parameters.noiseoptions.dy,i,j);
    }

  //fprintf(stderr, "noise: %f %f %f \n", x, y, sum);
  return(sum);

}
/* end of stolen code	;-) */
