/*********************************************************************************/
/* LIC 0.50 -- image filter plug-in for The Gimp program                         */
/* Copyright (C) 2001/02 Frank Lauterwald                                        */
/*===============================================================================*/
/* E-mail: frankl@gmx.net                                                        */
/* You can contact the original The Gimp authors at gimp@xcf.berkeley.edu        */
/*===============================================================================*/
/* This program is free software; you can redistribute it and/or modify it under */
/* the terms of the GNU General Public License as published by the Free Software */
/* Foundation; either version 2 of the License, or (at your option) any later    */
/* version.                                                                      */
/*===============================================================================*/
/* This program is distributed in the hope that it will be useful, but WITHOUT   */
/* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS */
/* FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.*/
/*===============================================================================*/
/* You should have received a copy of the GNU General Public License along with  */
/* this program; if not, write to the Free Software Foundation, Inc., 675 Mass   */
/* Ave, Cambridge, MA 02139, USA.                                                */
/*===============================================================================*/
/* In other words, you can't sue me for whatever happens while using this ;)     */
/*********************************************************************************/
/* CHANGELOG									 */
/* Date		Version		Comment						 */
/* 2001 Oct. 1,    0.30		first "release"					 */
/* 2001 Oct  4     0.32		changed the plugin-name as shown in gimp-menu    */
/* 2002 Jan. 26,   0.50		release of first beta version			 */
/*********************************************************************************/


#define GIMP_ENABLE_COMPAT_CRUFT
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <libgimp/gimp.h>
#include <libgimp/gimpui.h>

#include <libgimp/gimpenums.h>

#include "lic_globals.h"
#include "lic_plugin.h"


/*****************************************/
/* The Plugin-Info-Structure for telling */
/* The Gimp which functions to call	 */
/*****************************************/
GPlugInInfo PLUG_IN_INFO =
{
  NULL,    /* init_proc */
  NULL,    /* quit_proc */
  query,   /* query_proc */
  run,     /* run_proc   */
};



/* Macro defined by The Gimp. This tells The Gimp about our   */
/* PLUGIN_INFO. Then The Gimp knows our function pointers     */
/* and calls query(). query() then does the actual registering*/
MAIN()

/* used for registering with The Gimp */
static void query(void)
{
  static GParamDef args[] =
    {
      { PARAM_INT32, "run_mode", "Interactive" },
      { PARAM_IMAGE, "image", "Input image" },
      { PARAM_DRAWABLE, "drawable", "Input drawable" },
    };

  static GParamDef *return_vals = NULL;
  static gint nargs = sizeof (args) / sizeof (args[0]);
  static gint nreturn_vals = 0;

  gimp_install_procedure (PLUGINNAME,
			  "Creates a Van Gogh effect (Line Integral Convolution)",
			  "See http://studhome.rrze.uni-erlangen.de/~sifrlaut/",
			  "Frank Lauterwald",
			  "Frank Lauterwald",
			  "Version 0.50, January 26 2002",
			  "<Image>/Filters/Artistic/Van Gogh (Lic2)",
			  "RGB, GRAY",
			  PROC_PLUG_IN,
			  nargs, nreturn_vals,
			  args, return_vals);
}

static void run(gchar   *name,
                gint     nparams,
                GParam  *param,
                gint    *nreturn_vals,
                GParam **return_vals)
{
  static GimpParam values[1];
  GimpDrawable *drawable;
  GRunModeType run_mode;
  GStatusType status = STATUS_SUCCESS;


  run_mode = param[0].data.d_int32;

  *nreturn_vals = 1;
  *return_vals = values;

  values[0].type = PARAM_STATUS;
  values[0].data.d_status = status;


  /* Possibly retrieve data */
  /* ====================== */

  /*FIXME gimp_get_data (PLUGINNAME, &parameters); */

  /* Get the specified drawable */
  /* ========================== */

  drawable = gimp_drawable_get (param[2].data.d_drawable);


  if (status == STATUS_SUCCESS)
    {
          switch (run_mode)
            {
              case RUN_INTERACTIVE:
		VOUT("running lic_gimp_interactive from desired position \n");
		lic_gimp_interactive(*drawable);
                gimp_set_data(PLUGINNAME, &parameters, sizeof(parameters));
              break;
              case RUN_WITH_LAST_VALS:
                /*image_setup(drawable,FALSE);*/
		/* ALSO MUSS IMAGE_SETUP WIEDER EINE EIGENE FKT. WERDEN, DA DIESE FR MEHRERE CASES BENTIGT WIRD */
                /*lic_compute_image(); */
                break;
              default:
                break;
            }
        }
      else
        status = STATUS_EXECUTION_ERROR;

  values[0].data.d_status = status;
  gimp_drawable_detach (drawable);
  VOUT("LIC: work done (@run) \n");
}


