/*********************************************************************************/
/* LIC 0.50 -- image filter plug-in for The Gimp program                         */
/* Copyright (C) 2001/02 Frank Lauterwald                                        */
/*===============================================================================*/
/* E-mail: frankl@gmx.net                                                        */
/* You can contact the original The Gimp authors at gimp@xcf.berkeley.edu        */
/*===============================================================================*/
/* This program is free software; you can redistribute it and/or modify it under */
/* the terms of the GNU General Public License as published by the Free Software */
/* Foundation; either version 2 of the License, or (at your option) any later    */
/* version.                                                                      */
/*===============================================================================*/
/* This program is distributed in the hope that it will be useful, but WITHOUT   */
/* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS */
/* FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.*/
/*===============================================================================*/
/* You should have received a copy of the GNU General Public License along with  */
/* this program; if not, write to the Free Software Foundation, Inc., 675 Mass   */
/* Ave, Cambridge, MA 02139, USA.                                                */
/*===============================================================================*/
/* In other words, you can't sue me for whatever happens while using this ;)     */
/*********************************************************************************/
/* CHANGELOG									 */
/* Date		Version		Comment						 */
/* 2001 Oct. 1,    0.30		first "release"					 */
/* 2002 Jan. 26,   0.50		release of first beta version			 */
/*********************************************************************************/

/* Most of the userinterface is designed with glade. Glade creates a file main.c
 * I copied the needed parts of main() from main.c here and created a header for
 * it so it can be used by my other files

 * I tried to put all changes here so the glade-generated files can remain
 * as unchanged as possible
 *
 */


#include <stdio.h>

#include <gtk/gtk.h>

#include "interface.h"
#include "support.h"
#include "callbacks.h"

#include "../lic_globals.h"
#include "../filterfunction/lic_filter.h"

// this widget must be accessible by different functions
GtkWidget *window_mainwindow;

// non-exported function declarations
void userinterface_add_nonglade_widgets();
void userinterface_set_value_check_create_newimage();
void userinterface_set_value_method();
void userinterface_set_value_algorithm();
void userinterface_set_value_filter();
void userinterface_set_value_channel();
void userinterface_set_value_rotate();
void userinterface_set_value_convolvewith();
void userinterface_set_value_filtercurve();

/* Main UI function. It calls the stuff provided by glade output
 */
void show_userinterface()
{	int number=0;
	/* this is a necessary hack */
	gtk_init(&number, NULL);

	window_mainwindow = create_window_mainwindow();
	userinterface_add_nonglade_widgets();
	userinterface_set_values();

	gtk_widget_show(window_mainwindow);
	userinterface_set_statusbar("");

	userinterface_set_values();
	gtk_main();
	VOUT("gtk_main started \n ");
}

/* this function sets a statusbar text
 * Every action in the UI (i.e. each callback function)
 * That does not have anything to report (i.e. no problems)
 * Shall set the statusbar to "" to make sure that an error
 * message doesnt hang around all day
 */
void userinterface_set_statusbar(gchar *text) {
	GtkStatusbar *st = (GTK_STATUSBAR(lookup_widget(window_mainwindow, "statusbar_options")));
	gtk_statusbar_pop(st, 1);
	gtk_statusbar_push(st, 1, text);
}

/* we have to add some widgets manually
 * e.g. glade does not know adjustments,
 * so instead of messing around with glade output,
 * causing problems when changing the ui with glade
 * I add the additional stuff that I need here
 */
void userinterface_add_nonglade_widgets() {
	GtkAdjustment *adj;
	GtkGammaCurve *curve;
	GtkWidget *wid;
	gfloat points[4];
	// first the image menu
	gtk_option_menu_set_menu(GTK_OPTION_MENU(lookup_widget(window_mainwindow, "optionmenu_effectimage")),
  				gimp_drawable_menu_new(effect_image_constrain,
                                effect_image_callback,
				NULL,
                                parameters.options.vectorImageDrawable));

	// then everything that has to do with adjustments ;-)
	// first filterlength adjustment
	adj = GTK_ADJUSTMENT(gtk_adjustment_new(parameters.filterlength, 0.1, MAXFILTERLEN, 0.1, 1.0, 0.0));
	gtk_spin_button_configure(GTK_SPIN_BUTTON(lookup_widget(window_mainwindow, "spin_filterlength")), adj , 0.1, 2);
	gtk_object_set(GTK_OBJECT(lookup_widget(window_mainwindow, "hscale_filterlength")), "adjustment", adj, NULL);
	gtk_signal_connect (GTK_OBJECT(adj), "value_changed",
  		      GTK_SIGNAL_FUNC (on_filterlength_changed),
		      NULL);
	// -- first filterlength adjustment

	// then magnitude
	adj = GTK_ADJUSTMENT(gtk_adjustment_new(parameters.noiseoptions.magnitude, 1.0, 5.0, 0.1, 1.0, 0.0));
	gtk_spin_button_configure(GTK_SPIN_BUTTON(lookup_widget(window_mainwindow, "spin_noisemagnitude")), adj , 0.1, 2);
	gtk_object_set(GTK_OBJECT(lookup_widget(window_mainwindow, "hscale_noisemagnitude")), "adjustment", adj, NULL);
	gtk_signal_connect (GTK_OBJECT(adj), "value_changed",
  		      GTK_SIGNAL_FUNC (on_noisemagnitude_changed),
		      NULL);
	// -- then noise magnitude

	// then noise minimum value
	adj = GTK_ADJUSTMENT(gtk_adjustment_new(parameters.noiseoptions.minv, -100.0, 0.0, 1.0, 10.0, 0.0));
	gtk_spin_button_configure(GTK_SPIN_BUTTON(lookup_widget(window_mainwindow, "spin_minv")), adj , 0.1, 2);
	gtk_object_set(GTK_OBJECT(lookup_widget(window_mainwindow, "hscale_minv")), "adjustment", adj, NULL);
	gtk_signal_connect (GTK_OBJECT(adj), "value_changed",
  		      GTK_SIGNAL_FUNC (on_minv_changed),
		      NULL);
	// -- then noise minimum value

	// then noise maximum value
	adj = GTK_ADJUSTMENT(gtk_adjustment_new(parameters.noiseoptions.maxv, 0.0, 100.0, 1.0, 10.0, 0.0));
	gtk_spin_button_configure(GTK_SPIN_BUTTON(lookup_widget(window_mainwindow, "spin_maxv")), adj , 0.1, 2);
	gtk_object_set(GTK_OBJECT(lookup_widget(window_mainwindow, "hscale_maxv")), "adjustment", adj, NULL);
	gtk_signal_connect (GTK_OBJECT(adj), "value_changed",
  		      GTK_SIGNAL_FUNC (on_maxv_changed),
		      NULL);
	// -- then noise maximum value


	// finally stepsize
	adj = GTK_ADJUSTMENT(gtk_adjustment_new(parameters.algorithm.extras.ddaEuler.stepsize, 0.01, 1.5, 0.01, 0.1, 0.0));
	gtk_spin_button_configure(GTK_SPIN_BUTTON(lookup_widget(window_mainwindow, "spin_stepsize")), adj , 0.1, 2);
	gtk_object_set(GTK_OBJECT(lookup_widget(window_mainwindow, "scale_stepsize")), "adjustment", adj, NULL);
	gtk_signal_connect (GTK_OBJECT(adj), "value_changed",
  		      GTK_SIGNAL_FUNC (on_stepsize_changed),
		      NULL);
	// -- finally stepsize

	// add a curve to frame_filtercurve
	wid = lookup_widget(window_mainwindow, "frame_filtercurve");
	curve = GTK_GAMMA_CURVE(gtk_gamma_curve_new());
	gtk_widget_show (GTK_WIDGET(curve));
  	gtk_container_add (GTK_CONTAINER(wid), GTK_WIDGET(curve));
	//gtk_curve_set_range(curve, -1, 1, 0, 1);
	//gtk_curve_set_gamma(curve, 1.0);
	gtk_object_set_data_full (GTK_OBJECT (window_mainwindow), "curve_filtercurve", curve,
                            (GtkDestroyNotify) gtk_widget_unref);

	// -- add a curve to frame_filtercurve
}



/* this function sets all adjustable widgets to appropriate values
 * Think of a button selects a preset for all adjustable option.
 * If you press it, all widgets showing these options must be
 * informed to display the new values.
 * This can be done by calling this function.
 * DANGER: call this only from callbacks of widgets which are not in turn changed
 * by this function or infinite loops may be caused
 * This function only calls short helpers that each adjust the value
 * of a single widget.
 */
void userinterface_set_values() {

	userinterface_set_value_check_create_newimage();
	userinterface_set_value_method();
	userinterface_set_value_algorithm();
	userinterface_set_value_filter();
	userinterface_set_value_rotate();
	userinterface_set_value_channel();
	userinterface_set_value_convolvewith();
	userinterface_set_value_filtercurve();
}


void userinterface_set_value_check_create_newimage() {
	// create new image toggle
	if (parameters.options.createNewImage) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget(window_mainwindow, "check_create_newimage")), TRUE);
	}
	else {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget(window_mainwindow, "check_create_newimage")), FALSE);
	}
	// -- create new image toggle
}

void userinterface_set_value_method() {
	// visualization method (dda/lic)
	if ( (parameters.algorithm.algorithm==ALGORITHM_DDAEULER) || (parameters.algorithm.algorithm==ALGORITHM_DDACABRAL)) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget(window_mainwindow, "radio_method_dda")), TRUE);
	} else {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget(window_mainwindow, "radio_method_lic")), TRUE);
	}
	// -- visualization method (dda/lic)
}

void userinterface_set_value_algorithm() {
	// algorithm (euler / cabral)
	if ( (parameters.algorithm.algorithm==ALGORITHM_DDAEULER) || (parameters.algorithm.algorithm==ALGORITHM_LICEULER)) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget(window_mainwindow, "radio_algorithm_euler")), TRUE);
	} else {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget(window_mainwindow, "radio_algorithm_cabral")), TRUE);
	}
	// -- algorithm (euler / cabral)
}


void userinterface_set_value_filter() {
	if ( parameters.filterfunction.filterfunction == FILTERFUNCTION_BOX) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget(window_mainwindow, "radio_filter_box")), TRUE); }
	if ( parameters.filterfunction.filterfunction == FILTERFUNCTION_SMOOTHBOX) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget(window_mainwindow, "radio_filter_smoothbox")), TRUE); }
	if ( parameters.filterfunction.filterfunction == FILTERFUNCTION_TRIANGLE) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget(window_mainwindow, "radio_filter_triangle")), TRUE); }
	if ( parameters.filterfunction.filterfunction == FILTERFUNCTION_ASTROID) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget(window_mainwindow, "radio_filter_astroid")), TRUE); }
	if ( parameters.filterfunction.filterfunction == FILTERFUNCTION_OTHER) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget(window_mainwindow, "radio_filter_other")), TRUE); }
}

void userinterface_set_value_channel() {
	if ( parameters.options.effectchannel == EFFECTCHANNEL_HUE) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget(window_mainwindow, "radio_effectchannel_hue")), TRUE); }
	if ( parameters.options.effectchannel == EFFECTCHANNEL_SATURATION) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget(window_mainwindow, "radio_effectchannel_saturation")), TRUE); }
	if ( parameters.options.effectchannel == EFFECTCHANNEL_BRIGHTNESS) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget(window_mainwindow, "radio_effectchannel_brightness")), TRUE); }
	if ( parameters.options.effectchannel == EFFECTCHANNEL_RED) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget(window_mainwindow, "radio_effectchannel_red")), TRUE); }
	if ( parameters.options.effectchannel == EFFECTCHANNEL_GREEN) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget(window_mainwindow, "radio_effectchannel_green")), TRUE); }
	if ( parameters.options.effectchannel == EFFECTCHANNEL_BLUE) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget(window_mainwindow, "radio_effectchannel_blue")), TRUE); }
	if ( parameters.options.effectchannel == EFFECTCHANNEL_ALPHA) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget(window_mainwindow, "radio_effectchannel_alpha")), TRUE); }
}

void userinterface_set_value_rotate() {
	if ( parameters.options.rotateVectors) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget(window_mainwindow, "radio_rotatevectors_yes")), TRUE);
	} else {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget(window_mainwindow, "radio_rotatevectors_no")), TRUE);
	}

}


void userinterface_set_value_convolvewith() {
	if (parameters.options.getTextureFrom == GETTEXTUREFROM_NOISE) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget(window_mainwindow, "radio_convolvewith_noise")), TRUE);
	} else {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget(window_mainwindow, "radio_convolvewith_image")), TRUE);
	}
}

void userinterface_set_value_filtercurve() {
	gfloat points[FILTERCURVE_NUMBER_OF_POINTS];
	gint i;
	filterfunction 		filter;
	GtkGammaCurve *curve = GTK_GAMMA_CURVE(lookup_widget(window_mainwindow, "curve_filtercurve"));
	switch(parameters.filterfunction.filterfunction) {
		case FILTERFUNCTION_BOX:
			filter = filter_blockfilter;
			break;
		case FILTERFUNCTION_SMOOTHBOX:
			filter = filter_smoothblockfilter;
			break;
		case FILTERFUNCTION_TRIANGLE:
			filter = filter_trianglefilter;
			break;
		case FILTERFUNCTION_ASTROID:
			filter = filter_astroidfilter;
			break;
		case FILTERFUNCTION_OTHER:
			return;
	}
	for (i=0; i<FILTERCURVE_NUMBER_OF_POINTS; i++) {
		points[i] = filter( (i*1.0-FILTERCURVE_NUMBER_OF_POINTS/2)/FILTERCURVE_NUMBER_OF_POINTS*parameters.filterlength*2);
	}
	gtk_curve_set_vector(GTK_CURVE(curve->curve), FILTERCURVE_NUMBER_OF_POINTS, points);
}

// lookup a widget by name beginning at our mainwindow
GtkWidget *userinterface_get_widget(gchar *name) {
	return lookup_widget(window_mainwindow, name);
}
