/*  Physically Modeled Media Plug-In for The GIMP
 *  Copyright (c) 2000-2001 David A. Bartold
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __COLORPENCIL_H__
#define __COLORPENCIL_H__


#include "gtkrgbarea.h"


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define TYPE_COLOR_PENCIL            (color_pencil_get_type ())
#define COLOR_PENCIL(obj)            (GTK_CHECK_CAST ((obj), TYPE_COLOR_PENCIL, ColorPencil))
#define COLOR_PENCIL_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), TYPE_COLOR_PENCIL, ColorPencilClass))
#define IS_COLOR_PENCIL(obj)         (GTK_CHECK_TYPE ((obj), TYPE_COLOR_PENCIL))
#define IS_COLOR_PENCIL_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), TYPE_COLOR_PENCIL))


typedef struct _PencilPath        PencilPath;

struct _PencilPath
{
  gboolean eraser;
  gboolean no_last_point;
  gfloat   last_x;
  gfloat   last_y;
  gfloat   last_pressure;
  gfloat   blot_x;
  gfloat   blot_y;
};

typedef struct _ColorPencil      ColorPencil;
typedef struct _ColorPencilClass ColorPencilClass;

struct _ColorPencil
{
  GtkRgbArea    parent;

  guint8       *wax;
  guint8       *texture;

  gfloat        point_red;
  gfloat        point_green;
  gfloat        point_blue;
  gfloat        point_radius;
  gfloat        point_opacity;
  gfloat        point_waxiness;

  gfloat        eraser_radius;
  gfloat        eraser_red;
  gfloat        eraser_green;
  gfloat        eraser_blue;
};

struct _ColorPencilClass
{
  GtkRgbAreaClass parent_class;
};


GtkType     color_pencil_get_type         (void);
GtkWidget  *color_pencil_new              (void);
void        color_pencil_point_color      (ColorPencil *pencil,
                                           gfloat       red,
                                           gfloat       green,
                                           gfloat       blue);
void        color_pencil_point_properties (ColorPencil *pencil,
                                           gfloat       point_radius,
                                           gfloat       point_opacity,
                                           gfloat       point_waxiness);
void        color_pencil_eraser_color     (ColorPencil *pencil,
                                           gfloat       red,
                                           gfloat       green,
                                           gfloat       blue);
void        color_pencil_point            (ColorPencil *pencil,
                                           gfloat       center_x,
                                           gfloat       center_y,
                                           gfloat       pressure);
void        color_pencil_erase            (ColorPencil *pencil,
                                           gfloat       center_x,
                                           gfloat       center_y,
                                           gfloat       pressure);
void        color_pencil_clear            (ColorPencil *pencil);
PencilPath *color_pencil_path_start       (ColorPencil *pencil,
                                           gboolean     eraser);
void        color_pencil_path_point       (ColorPencil *pencil,
                                           PencilPath  *path,
                                           gfloat       x,
                                           gfloat       y,
                                           gfloat       pressure);
void        color_pencil_path_end         (ColorPencil *pencil,
                                           PencilPath  *path);

#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* __COLORPENCIL_H__ */
