#include <gtk/gtk.h>
#include "support.h"
#include "support2.h"
#include "colorpicker.h"
#include "gtkrgbarea.h"

static void
update (GtkWidget *swatch)
{
  gdouble    *color;
  GtkRgbArea *area;
  gint        i, r, g, b;

  color = gtk_object_get_data (GTK_OBJECT (swatch), "data_color");
  area = gtk_object_get_data (GTK_OBJECT (swatch), "area");

  r = color[0] * 255.0;
  g = color[1] * 255.0;
  b = color[2] * 255.0;
  for (i = 0; i < area->width * area->height * 3; i += 3)
    {
      area->rgb_data[i + 0] = r;
      area->rgb_data[i + 1] = g;
      area->rgb_data[i + 2] = b;
    }
}

static void
color_area_sel_destroy (GtkWidget *area)
{
  gtk_object_remove_data (GTK_OBJECT (area), "color_sel");
  gtk_object_unref (GTK_OBJECT (area));
}


static void
ok_clicked (GtkWidget *button,
            GtkWidget *dialog)
{
  gdouble    colors[3];
  GtkWidget *swatch;
  GtkWidget *color_sel;

  swatch = lookup_data (GTK_WIDGET (dialog), "swatch");
  color_sel = GTK_COLOR_SELECTION_DIALOG (dialog)->colorsel;
  gtk_color_selection_get_color (GTK_COLOR_SELECTION (color_sel), colors);

  color_picker_set_color (swatch, colors[0], colors[1], colors[2]);

  gtk_widget_destroy (dialog);
}


static void
cancel_clicked (GtkWidget *button,
                GtkWidget *dialog)
{
  gtk_widget_destroy (dialog);
}


static void
button_clicked (GtkWidget      *button,
                GdkEventExpose *expose)
{
  gdouble   *color;
  GtkWidget *color_sel_dialog;
  GtkWidget *color_sel;

  color = gtk_object_get_data (GTK_OBJECT (button), "data_color");
  color_sel_dialog = gtk_object_get_data (GTK_OBJECT (button), "color_sel");
  if (color_sel_dialog == NULL)
    {
      color_sel_dialog = gtk_color_selection_dialog_new (_("Choose a color"));
      gtk_signal_connect (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (color_sel_dialog)->ok_button), "clicked",
                          GTK_SIGNAL_FUNC (&ok_clicked), color_sel_dialog);
      gtk_signal_connect (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (color_sel_dialog)->cancel_button), "clicked",
                          GTK_SIGNAL_FUNC (&cancel_clicked), color_sel_dialog);
      gtk_widget_hide (
        GTK_COLOR_SELECTION_DIALOG (color_sel_dialog)->help_button);
      gtk_object_set_data (GTK_OBJECT (button), "color_sel", color_sel_dialog);

      gtk_object_ref (GTK_OBJECT (button));
      gtk_object_set_data_full (GTK_OBJECT (color_sel_dialog), "swatch",
                                button,
                                (GtkDestroyNotify) color_area_sel_destroy);
  }

  color_sel = GTK_COLOR_SELECTION_DIALOG (color_sel_dialog)->colorsel;
  gtk_color_selection_set_color (GTK_COLOR_SELECTION (color_sel), color);

  gtk_widget_show (color_sel_dialog);
  gdk_window_raise (color_sel_dialog->window);
}


static void
realize (GtkWidget *area,
         GtkWidget *button)
{
  update (button);
}


GtkWidget *
color_picker_new (void)
{
  GtkWidget *button;
  GtkWidget *area;
  gdouble   *color;

  button = gtk_button_new ();
  area = gtk_rgb_area_new ();
  gtk_drawing_area_size (GTK_DRAWING_AREA (area), 40, 30);
  gtk_signal_connect (GTK_OBJECT (area), "realize",
                      GTK_SIGNAL_FUNC (&realize), button);
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      GTK_SIGNAL_FUNC (&button_clicked), NULL);

  gtk_widget_show (area);
  gtk_container_add (GTK_CONTAINER (button), area);

  color = g_new0 (gdouble, 3);
  gtk_object_set_data_full (GTK_OBJECT (button), "data_color",
                            color, (GtkDestroyNotify) g_free);
  gtk_object_ref (GTK_OBJECT (area));
  gtk_object_set_data_full (GTK_OBJECT (button), "area",
                            area, (GtkDestroyNotify) gtk_object_unref);

  return button;
}


void
color_picker_get_color (GtkWidget *widget,
                        gdouble   *red,
                        gdouble   *green,
                        gdouble   *blue)
{
  gdouble *color;

  color = gtk_object_get_data (GTK_OBJECT (widget), "data_color");
  *red = color[0];
  *green = color[1];
  *blue = color[2];
}


void
color_picker_set_color (GtkWidget *widget,
                        gdouble    red,
                        gdouble    green,
                        gdouble    blue)
{
  GtkRgbArea *area;
  gdouble    *color;

  color = gtk_object_get_data (GTK_OBJECT (widget), "data_color");
  area = gtk_object_get_data (GTK_OBJECT (widget), "area");

  color[0] = red;
  color[1] = green;
  color[2] = blue;

  update (widget);

  gtk_widget_queue_draw (GTK_WIDGET (area));
}
