/*  Physically Modeled Media Plug-In for The GIMP
 *  Copyright (c) 2000-2001 David A. Bartold
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <libgimp/gimp.h>
#include <libgimp/gimpui.h>

#include "interface.h"
#include "support.h"
#include "gimpsupport.h"
#include "colorpencil.h"
#include "texture.h"



/* Originally from Raph Levien's wet dream program */
static void
input_init (void)
{
  GList *tmp_list;
  GdkDeviceInfo *info;

  tmp_list = gdk_input_list_devices();

  info = NULL;
  while (tmp_list)
    {
      info = (GdkDeviceInfo *)tmp_list->data;

      if (info->source == GDK_SOURCE_PEN ||
          info->source == GDK_SOURCE_ERASER)
        {
          g_print ("Setting '%s' as screen device.\n", info->name);
          gdk_input_set_mode (info->deviceid, GDK_MODE_SCREEN);
        }

      tmp_list = tmp_list->next;
    }
}

gint
pencil_window (gint32        image_id,
               GimpDrawable *drawable)
{
  gint32           channel_id;
  GimpDrawable    *texture_drawable;
  GtkWidget       *window;
  GtkWidget       *pencil;
  gint             width, height;
  gint             offset_x, offset_y;
  gint             texture_width, texture_height;
  int              argc = 1;
  char            *argv[2] = { "digi_pencil", NULL };
  char           **argv2 = argv;

#ifdef ENABLE_NLS
  bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
  textdomain (PACKAGE);
#endif

  gtk_set_locale ();
  setlocale (LC_NUMERIC, "C");
  gtk_init (&argc, &argv2);
  gdk_rgb_init ();

  gtk_widget_set_default_colormap (gdk_rgb_get_cmap ());
  gtk_widget_set_default_visual (gdk_rgb_get_visual ());

  input_init ();

  add_pixmap_directory (g_strconcat (PACKAGE_DATA_DIR, G_DIR_SEPARATOR_S "pixmaps", NULL));
  add_pixmap_directory (g_strconcat (PACKAGE_SOURCE_DIR, G_DIR_SEPARATOR_S "pixmaps", NULL));

  width = gimp_drawable_width (drawable->id);
  height = gimp_drawable_height (drawable->id);

  window = create_pencil_window ();

  gtk_object_set_data (GTK_OBJECT (window), "data_image_id", &image_id);
  gtk_object_set_data (GTK_OBJECT (window), "data_drawable", drawable);

  pencil = lookup_widget (window, "color_pencil");
  gtk_drawing_area_size (GTK_DRAWING_AREA (pencil), width, height);

  gtk_widget_show_all (window);
  gtk_widget_realize (window);
  gdk_window_resize (window->window,
                     MAX (MIN ((gint) (gdk_screen_width () * 3 / 4),
                               width + 20),
                          window->allocation.width),
                     MAX (MIN ((gint) (gdk_screen_height () * 3 / 4),
                               window->allocation.height - 55 + height),
                          window->allocation.height));

  gimp_drawable_offsets (drawable->id, &offset_x, &offset_y);

  /* Load the texture */
  channel_id = find_channel (image_id, "Texture");
  if (channel_id >= 0)
    {
      texture_drawable = gimp_drawable_get (channel_id);
      texture_width = gimp_drawable_width (channel_id);
      texture_height = gimp_drawable_height (channel_id);
      tiles_to_from_pixmap (COLOR_PENCIL (pencil)->texture,
                            1, width, texture_width, texture_height,
                            texture_drawable, -offset_x, -offset_y,
                            TRUE);
      gimp_drawable_detach (texture_drawable);
    }
  else
    {
      /* Otherwise autogenerate a default texture. */
      g_free (COLOR_PENCIL (pencil)->texture);
      COLOR_PENCIL (pencil)->texture =
        texture_light_new (width, height, 1.0, 0);
    }

  /* Load the wax channel */
  channel_id = find_channel (image_id, "Wax");
  if (channel_id >= 0)
    {
      texture_drawable = gimp_drawable_get (channel_id);
      texture_width = gimp_drawable_width (channel_id);
      texture_height = gimp_drawable_height (channel_id);
      tiles_to_from_pixmap (COLOR_PENCIL (pencil)->wax,
                            1, width, texture_width, texture_height,
                            texture_drawable, -offset_x, -offset_y,
                            TRUE);
      gimp_drawable_detach (texture_drawable);
    }

  /* Load the color layer */
  tiles_to_from_pixmap (GTK_RGB_AREA (pencil)->rgb_data,
                        3, width * 3, width, height,
                        drawable, 0, 0,
                        TRUE);

  gtk_main ();
  gimp_image_set_active_layer (image_id, drawable->id);
  gimp_displays_flush ();
  gdk_flush ();

  return 0;
}

