#include "perlintexture.h"

struct _PerlinTexture
{
  TPerlin *perlin;
  gint     type;
  gfloat   scale;
};

PerlinTexture *
perlin_texture_new ()
{
  PerlinTexture *texture;

  texture = g_new0 (PerlinTexture, 1);
  texture->perlin = t_perlin_new (10, 2.0, 0.5, 0);
  texture->type = 0;
  texture->scale = 1.0;

  return texture;
}

void
perlin_texture_free (PerlinTexture *texture)
{
  if (texture == NULL)
    return;

  t_perlin_free (texture->perlin);
  g_free (texture);
}

void
perlin_texture_properties (PerlinTexture *texture,
                           gint           type,
                           gfloat         scale)
{
  texture->type = type;
  texture->scale = 1.0 / scale;
}

gfloat
perlin_texture_get (PerlinTexture *texture,
                    gint           x,
                    gint           y)
{
  switch (texture->type)
    {
    case 0:
      return t_perlin_get (texture->perlin,
                           1000.0 + x * texture->scale * 0.2,
                           1000.0 + y * texture->scale);
      break;

    case 1:
      return t_perlin_get (texture->perlin,
                           1000.0 + x * texture->scale,
                           1000.0 + y * texture->scale);
      break;
    }

  return 0.0;
}

guint8 *
make_perlin_texture (gint   width,
                     gint   height,
                     gint   type,
                     gfloat scale,
                     gint   seed)
{
  PerlinTexture *texture;
  gint           x, y, pos;
  guint8        *data;

  texture = perlin_texture_new ();
  perlin_texture_properties (texture, type, scale);
  data = g_new0 (guint8, width * height);
  pos = 0;
  for (y = 0; y < height; y++)
    for (x = 0; x < width; x++)
      data[pos++] = perlin_texture_get (texture, x, y) * 255.0;

  perlin_texture_free (texture);

  return data;
}

guint8 *
perlin_texture_light_new (gint   width,
                          gint   height,
                          gfloat scale,
                          gint   seed)
{
  return make_perlin_texture (width, height, 0, scale, seed);
}

guint8 *
perlin_texture_heavy_new (gint   width,
                          gint   height,
                          gfloat scale,
                          gint   seed)
{
  return make_perlin_texture (width, height, 1, scale, seed);
}
